from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404

from .models import Category, Forum, Topic


class TopicFeed(Feed):
    # attr basic of feed
    title = 'Forum rss'
    description = 'Feed for forums'

    def get_object(self, request, *args, **kwargs):
        category = get_object_or_404(Category, name=kwargs['category'])
        forum = get_object_or_404(
            Forum, category__name=category, name=kwargs['forum']
        )
        return forum

    def items(self, forum):
        # Get forum
        return Topic.objects.filter(forum_id=forum.idforum)

    def item_link(self, item):
        return reverse("topic", args=[
            item.forum.category.name, item.forum.name, item.slug, item.idtopic
        ])

    def link(self, forum):
        return reverse('rss', args=[
            forum.category.name, forum.name
        ])

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.description

    def item_pubdate(self, item):
        return item.date



from django import forms
from django.conf import settings
from django.contrib.auth import (
    authenticate, get_user_model, password_validation
)
from django.contrib.staticfiles import finders
from django.forms.widgets import ClearableFileInput, CheckboxInput
from django.utils import timezone
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from musette import models, utils, widgets
from musette.email import send_mail


class FormLogin(forms.Form):
    """
    Form of login
    """
    widgetUser = forms.TextInput(attrs={
        'class': 'form-control', 'placeholder': _("Username")
    })
    widgetPass = forms.PasswordInput(attrs={
        'class': 'form-control', 'placeholder': ("Password")
    })
    username = forms.CharField(
        max_length=45, widget=widgetUser, required=True
    )
    password = forms.CharField(
        max_length=45, widget=widgetPass, required=True
    )
    hidden_error = forms.CharField(required=False)

    def __init__(self, *args, **kwargs):
        super(FormLogin, self).__init__(*args, **kwargs)
        self.fields['username'].widget.attrs.update({'autofocus': ''})

    def form_authenticate(self):
        """
        This method if responsible of authenticate user login, if ok
        then return the user
        """
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')
        user = authenticate(username=username, password=password)

        if user is None:
            error = ""
            error = error + "<ul class='errorlist'><li>"
            error = error + str(_("Username or password incorrect."))
            error = error + "</li></ul>"
            self._errors["hidden_error"] = error

        return user


class FormSignUp(forms.ModelForm):
    """
    Form for create one new user
    """
    widget_pass = forms.PasswordInput(attrs={'class': 'form-control'})
    password = forms.CharField(max_length=128, widget=forms.PasswordInput)
    pass_confirm = forms.CharField(max_length=128, widget=widget_pass)

    class Meta:
        model = get_user_model()
        fields = [
            'username', 'email',
            'first_name', 'last_name',
            'password'
        ]

    def __init__(self, *args, **kwargs):
        super(FormSignUp, self).__init__(*args, **kwargs)
        self.fields['first_name'].widget.attrs.update({'autofocus': ''})
        class_css = 'form-control'

        for key in self.fields:
            self.fields[key].required = True
            self.fields[key].widget.attrs['class'] = class_css

            if key is "username":
                self.fields[key].widget.attrs['placeholder'] = _("Username")
            elif key is "email":
                self.fields[key].widget.attrs['placeholder'] = _("Email")
            elif key is "first_name":
                self.fields[key].widget.attrs['placeholder'] = _("Names")
            elif key is "last_name":
                self.fields[key].widget.attrs['placeholder'] = _("Surname")
            elif key is "password":
                self.fields[key].widget.attrs['placeholder'] = _("Password")
            elif key is "pass_confirm":
                placeholder = _("Repeat password")
                self.fields[key].widget.attrs['placeholder'] = placeholder

    # Valid the passwords
    def clean_pass_confirm(self):
        password1 = self.cleaned_data.get('password')
        password2 = self.cleaned_data.get('pass_confirm')
        if password1 and password1 != password2:
            raise forms.ValidationError(_("Passwords don't match"))

        # Check if password is very easy
        password_validation.validate_password(password2, self.instance)
        return password2

    # Valid the email that is unique
    def clean_email(self):
        email = self.cleaned_data.get('email')
        username = self.cleaned_data.get('username')

        User = get_user_model()
        count = User.objects.filter(
            email=email
        ).exclude(username=username).count()

        if email and count:
            raise forms.ValidationError(_('Email addresses must be unique.'))
        return email

    # Create one new user
    def create_user(self):
        username = self.cleaned_data.get('username')
        email = self.cleaned_data.get('email')
        first_name = self.cleaned_data.get('first_name')
        last_name = self.cleaned_data.get('last_name')
        password = self.cleaned_data.get('password')
        now = timezone.now()

        # Create user
        User = get_user_model()
        us = User(
            username=username, email=email,
            first_name=first_name,
            last_name=last_name, is_active=False,
            is_superuser=False, date_joined=now,
            is_staff=False
        )
        us.set_password(password)
        us.save()


class FormAdminTopic(forms.ModelForm):
    """
    Form for topic cadmin
    """
    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        super(FormAdminTopic, self).__init__(*args, **kwargs)

        if not self.request.user.is_superuser:
            queryset = models.Forum.objects.filter(
                moderators=self.request.user
            )
            self.fields['forum'].queryset = queryset

    class Meta:
        model = models.Topic
        exclude = ('slug', 'id_attachment')
        widgets = {
            'description': widgets.TextareaWidget,
        }


class FormAddTopic(forms.ModelForm):
    """
    Form for create one new topic
    """
    class Meta:
        model = models.Topic
        exclude = (
            "forum", "user", "slug", "date",
            "id_attachment", "moderate", "is_top"
            "is_close",
        )
        widgets = {
            'description': widgets.TextareaWidget,
        }

    def __init__(self, *args, **kwargs):
        super(FormAddTopic, self).__init__(*args, **kwargs)
        class_css = 'form-control'

        for key in self.fields:
            if key != "attachment":
                if key == "title" or key == "description":
                    self.fields[key].required = True
                    self.fields[key].widget.attrs['v-model'] = key
                    self.fields[key].widget.attrs['class'] = class_css
                    self.fields[key].widget.attrs['required'] = 'required'
            else:
                self.fields[key].required = False


class CustomClearableFileInput(ClearableFileInput):
    """
    Changes order fields
    """
    template_with_initial = (
        '%(initial_text)s: <a href="%(initial_url)s">%(initial)s</a> '
        '%(clear_template)s<br />%(input_text)s: %(input)s'
    )
    template_with_clear = ""
    template_with_clear = '<br>  <label for="%(clear_checkbox_id)s"> '
    template_with_clear += ' %(clear_checkbox_label)s</label> %(clear)s'

    def render(self, name, value, attrs=None):
        substitutions = {
            'initial_text': self.initial_text,
            'input_text': self.input_text,
            'clear_template': '',
            'clear_checkbox_label': self.clear_checkbox_label,
        }
        template = '%(input)s'
        substitutions['input'] = super(
            ClearableFileInput, self).render(name, value, attrs)

        if self.is_initial(value):
            template = self.template_with_initial
            substitutions.update(self.get_template_substitution_values(value))

            values = self.get_template_substitution_values(value)
            initial = utils.basename(values['initial'])

            if not self.is_required:
                checkbox_name = self.clear_checkbox_name(name)
                checkbox_id = self.clear_checkbox_id(checkbox_name)

                substitutions['clear_checkbox_name'] = conditional_escape(
                    checkbox_name)

                substitutions['clear_checkbox_id'] = conditional_escape(
                    checkbox_id)

                substitutions['clear'] = CheckboxInput().render(
                    checkbox_name, False, attrs={'id': checkbox_id})

                clear_template = self.template_with_clear % substitutions
                substitutions['clear_template'] = clear_template
                substitutions['initial'] = conditional_escape(initial)

        return mark_safe(template % substitutions)


class FormEditTopic(forms.ModelForm):
    """
    Form for edit one new topic
    """
    class Meta:
        model = models.Topic
        exclude = (
            "forum", "user", "slug", "date",
            "id_attachment", "moderate", "is_top",
            "is_close",
        )
        widgets = {
            'description': widgets.TextareaWidget,
            'attachment': CustomClearableFileInput,
        }

    def __init__(self, *args, **kwargs):
        super(FormEditTopic, self).__init__(*args, **kwargs)
        class_css = 'form-control'

        for key in self.fields:
            if key != "attachment":
                if key == "title" or key == "description":
                    v_init = ""
                    v_init += key + "=" + "'"
                    v_init += str(kwargs['instance'].title) + "'"
                    self.fields[key].required = True
                    self.fields[key].widget.attrs['v-model'] = key
                    self.fields[key].widget.attrs['value'] = v_init
                    self.fields[key].widget.attrs['class'] = class_css
                    self.fields[key].widget.attrs['required'] = 'required'
            else:
                self.fields[key].required = False


class FormAddComment(forms.ModelForm):
    """
    Form for add comment to topic
    """
    class Meta:
        model = models.Comment
        fields = ['description']
        widgets = {
            'description': widgets.TextareaWidget,
        }

    def __init__(self, *args, **kwargs):
        super(FormAddComment, self).__init__(*args, **kwargs)

        for key in self.fields:
            if key == "description":
                self.fields[key].required = True
                self.fields[key].widget.attrs['style'] = "width: 100%"
                self.fields[key].widget.attrs['v-model'] = key
                self.fields[key].widget.attrs['required'] = 'required'

    def send_mail_comment(self, url, list_email):
        # Send to email
        utils.send_mail_comment(url, list_email)


class FormAdminProfile(forms.ModelForm):
    """
    Form for admin profile
    """
    class Meta:
        model = utils.get_main_model_profile()
        exclude = (
            'idprofile', 'iduser',
        )
        widgets = {
            'about': widgets.TextareaWidget,
        }


class FormEditProfile(forms.ModelForm):
    """
    Form for edit one profile
    """
    class Meta:
        model = utils.get_main_model_profile()
        exclude = (
            'idprofile', 'iduser', 'activation_key',
            'key_expires', 'is_troll',
        )
        widgets = {
            'about': widgets.TextareaWidget,
            'photo': CustomClearableFileInput,
        }

    def __init__(self, *args, **kwargs):
        super(FormEditProfile, self).__init__(*args, **kwargs)
        class_css = 'form-control'

        for key in self.fields:
            self.fields[key].widget.attrs['class'] = class_css


class FormAdminConfiguration(forms.ModelForm):
    """
    Form configuration
    """
    name_file_custom = finders.find('musette/css/custom.css')

    class Meta:
        model = models.Configuration
        exclude = (
            'idconfig',
        )

    def __init__(self, *args, **kwargs):
        # Read css custom
        file_custom = open(self.name_file_custom, 'r')
        file_custom = file_custom.read()

        # Override init value on edit
        initial = kwargs.get('initial', {})
        initial['custom_css'] = file_custom
        kwargs['initial'] = initial
        super(FormAdminConfiguration, self).__init__(*args, **kwargs)

        # Init value to new record
        self.fields['custom_css'].initial = file_custom

    def save(self, commit=True):
        instance = super(FormAdminConfiguration, self).save(commit=False)

        # Save content in the file
        with open(self.name_file_custom, "w") as text_file:
            text_file.write(instance.custom_css)

        if commit:
            instance.save()
        return instance



import os

from django.conf import settings
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.cache import cache
from django.db import models
from django.shortcuts import get_object_or_404
from django.template import defaultfilters
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _

from . import settings as localSettings
from .validators import valid_extension


@python_2_unicode_compatible
class Category(models.Model):
    """
    Model Category
    """
    idcategory = models.AutoField(primary_key=True)
    name = models.CharField(max_length=80)
    position = models.IntegerField(blank=True, default=0)
    hidden = models.BooleanField(
        blank=False, null=False, default=False,
        help_text=_('If checked, this category will be visible only for staff')
    )

    class Meta(object):
        ordering = ['position']
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Forum(models.Model):
    """
    Model Forum
    """
    idforum = models.AutoField(primary_key=True)
    category = models.ForeignKey(
        Category, related_name='categories',
        verbose_name=_('Category'), on_delete=models.CASCADE
    )
    parent = models.ForeignKey(
        'self', related_name='parents', verbose_name=_('Parent forum'),
        blank=True, null=True, on_delete=models.CASCADE
    )
    name = models.CharField(_('Name'), max_length=80)
    position = models.IntegerField(_('Position'), blank=True, default=0)
    description = models.TextField(_('Description'), blank=True)
    moderators = models.ManyToManyField(
        settings.AUTH_USER_MODEL, related_name='moderators',
        verbose_name=_('Moderators')
    )
    date = models.DateTimeField(
        _('Date'), blank=True, null=True, auto_now_add=True, editable=False
    )
    topics_count = models.IntegerField(
        _('Topics count'), blank=True, default=0, editable=False,
    )
    hidden = models.BooleanField(
        _('Hidden'), blank=False, null=False, default=False,
        help_text=_('If hide the forum in the index page')
    )
    is_moderate = models.BooleanField(
        _('Check topics'), default=False,
        help_text=_('If the forum is moderated')
    )

    class Meta(object):
        unique_together = ('category', 'name', )
        ordering = ['category', 'position', 'name']
        verbose_name = _('Forum')
        verbose_name_plural = _('Forums')

    def __str__(self):
        return self.name

    # Return forums that moderating one moderator
    def tot_forums_moderators(self, moderator):
        tot = self.__class__.objects.filter(
            moderators=moderator
        ).count()

        return tot

    # Add permissions topic to moderator
    def add_permissions_topic_moderator(self, moderator):
        permission1 = Permission.objects.get(codename='add_topic')
        permission2 = Permission.objects.get(codename='change_topic')
        permission3 = Permission.objects.get(codename='delete_topic')

        moderator.user_permissions.add(permission1)
        moderator.user_permissions.add(permission2)
        moderator.user_permissions.add(permission3)

        # Add permission is_staff
        moderator.is_staff = True
        moderator.save()

    # Clear permissions to moderator
    def clear_permissions_moderator(self, moderator):
        moderator.user_permissions.clear()

        # Remove permission is_staff
        moderator.is_staff = False
        moderator.save()

    # Remove permission in user moderators
    def remove_user_permissions_moderator(self):
        for moderator in self.moderators.all():
            # Superuser not is necessary
            if not moderator.is_superuser:
                # Return forums that moderating one moderator
                tot_forum_moderator = self.tot_forums_moderators(moderator)

                # Only remove permissions if is moderator one forum
                if tot_forum_moderator < 1:
                    self.clear_permissions_moderator(moderator)

    def delete(self, *args, **kwargs):
        for moderator in self.moderators.all():
            if not moderator.is_superuser:
                # Only remove permissions if is moderator has one forum
                if self.tot_forums_moderators(moderator) < 1:
                    # Remove permissions to user
                    self.clear_permissions_moderator(moderator)

        super(Forum, self).delete()

    def clean(self):
        if self.name:
            self.name = self.name.strip()

    def forum_description(obj):
        return obj.description
    forum_description.allow_tags = True
    forum_description.short_description = _("Description")


class MessageForum(models.Model):
    """
    Message forum model
    """
    idmessage_forum = models.AutoField(primary_key=True)
    forum = models.ForeignKey(
        Forum, related_name='message_information', verbose_name=_('Forum'),
        on_delete=models.CASCADE
    )
    message_information = models.TextField(
        _('Message to inform'), blank=False, null=False,
        help_text=_('If you want to report a message to a forum')
    )
    message_expires_from = models.DateTimeField(
        _('Message expires from'), blank=False, null=False,
        help_text=_('Date from message expired')
    )
    message_expires_to = models.DateTimeField(
        _('Message expires to'), blank=False, null=False,
        help_text=_('Date to message expired')
    )

    class Meta(object):
        verbose_name = _('Message for forums')
        verbose_name_plural = _('Messages for forums')

    def __str__(self):
        return self.message_information


@python_2_unicode_compatible
class Topic(models.Model):
    """
    Model Topic
    """
    def generate_path(instance, filename):
        """
        Generate path to field Attchment
        """
        folder = ""
        folder = "forum_" + str(instance.forum_id)
        folder = folder + "_user_" + str(instance.user)
        folder = folder + "_topic_" + str(instance.id_attachment)
        return os.path.join("forum", folder, filename)

    idtopic = models.AutoField(primary_key=True)
    forum = models.ForeignKey(
        Forum, related_name='forums', verbose_name=_('Forum'),
        on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='users', verbose_name=_('User'),
        on_delete=models.CASCADE
    )
    slug = models.SlugField(max_length=100)
    title = models.CharField(_('Title'), max_length=80)
    date = models.DateTimeField(
        _('Date'), blank=False, auto_now=True, db_index=False
    )
    last_activity = models.DateTimeField(
        _('Last activity'), blank=False, auto_now=True, db_index=False
    )
    description = models.TextField(_('Description'), blank=False, null=False)
    id_attachment = models.CharField(max_length=200, null=True, blank=True)
    attachment = models.FileField(
        _('File'), blank=True, null=True, upload_to=generate_path,
        validators=[valid_extension]
    )
    is_close = models.BooleanField(
        _('Closed topic'), default=False,
        help_text=_('If the topic is closed')
    )
    moderate = models.BooleanField(
        _('Moderate'), default=False,
        help_text=_('If the topic is moderated')
    )
    is_top = models.BooleanField(
        _('Top'), default=False,
        help_text=_('If the topic is important and it will go top')
    )
    like = models.PositiveIntegerField(default=0, editable=False)

    class Meta(object):
        ordering = ['forum', 'last_activity', 'title', 'date']
        verbose_name = _('Topic')
        verbose_name_plural = _('Topics')

    def __str__(self):
        return self.title

    def delete(self, *args, **kwargs):
        idtopic = self.idtopic
        forum = self.forum_id

        topic = get_object_or_404(Topic, idtopic=idtopic)

        folder = ""
        folder = "forum_" + str(forum)
        folder = folder + "_user_" + str(topic.user.username)
        folder = folder + "_topic_" + str(topic.id_attachment)
        path_folder = os.path.join("forum", folder)
        media_path = settings.MEDIA_ROOT
        path = media_path + "/" + path_folder

        # Remove attachment if exists
        from .utils import remove_folder, exists_folder
        if exists_folder(path):
            remove_folder(path)

        Topic.objects.filter(idtopic=idtopic).delete()
        self.update_forum_topics(self.forum, "subtraction")

    def save(self, *args, **kwargs):

        if not self.idtopic:
            self.slug = defaultfilters.slugify(self.title)
            self.update_forum_topics(
                self.forum.category.name, self.forum, "sum"
            )

        self.moderate = self.check_topic_moderate()
        self.generate_id_attachment(self.id_attachment)
        super(Topic, self).save(*args, **kwargs)

    def update_forum_topics(self, category, forum, action):

        f = Forum.objects.get(category__name=category, name=forum)
        tot_topics = f.topics_count
        if action == "sum":
            tot_topics = tot_topics + 1
        elif action == "subtraction":
            tot_topics = tot_topics - 1

        Forum.objects.filter(name=forum).update(
            topics_count=tot_topics
        )

    def check_topic_moderate(self):
        """
        Check if one topic is mark like moderate
        """
        # If is superuser
        if self.user.is_superuser:
            return True
        # If the forum not is moderate
        elif not self.forum.is_moderate:
            return True
        # If user is moderator
        elif self.user in self.forum.moderators.all():
            return True
        else:
            return False

    def generate_id_attachment(self, value):
        if not value:
            self.id_attachment = get_random_string(length=32)


@python_2_unicode_compatible
class Comment(models.Model):
    """
    Model Comment
    """

    idcomment = models.AutoField(primary_key=True)
    topic = models.ForeignKey(
        Topic, related_name='topics', verbose_name=_('Topic'),
        on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='comment_users',
        verbose_name=_('User'), on_delete=models.CASCADE
    )
    date = models.DateTimeField(
        _('Date'), blank=True, auto_now=True, db_index=True
    )
    description = models.TextField(_('Description'), blank=True)
    like = models.PositiveIntegerField(default=0, editable=False)

    class Meta(object):
        ordering = ['date']
        verbose_name = _('Comment')
        verbose_name_plural = _('Comments')

    def __str__(self):
        return str(self.description)


@python_2_unicode_compatible
class Notification(models.Model):
    """
    Model Notification
    """
    idnotification = models.AutoField(primary_key=True)
    content_type = models.ForeignKey(
        ContentType, null=True, blank=True,
    )
    idobject = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey('content_type', 'idobject')
    iduser = models.IntegerField(default=0)
    is_topic = models.BooleanField(default=0)
    is_comment = models.BooleanField(default=0)
    is_view = models.BooleanField(default=0)
    date = models.DateTimeField(blank=True, db_index=True)

    class Meta(object):
        ordering = ['date']

    def __str__(self):
        return str(self.idnotification)


@python_2_unicode_compatible
class LikeTopic(models.Model):
    """
    Model LikeTopic
    """
    topic = models.ForeignKey(
        Topic, related_name='likes_topic', on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='likes_topic_users',
        on_delete=models.CASCADE
    )

    def __str__(self):
        return str(self.topic.idtopic)


@python_2_unicode_compatible
class LikeComment(models.Model):
    """
    Model LikeComment
    """
    comment = models.ForeignKey(
        Comment, related_name='likes_comment', on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='likes_comment_users',
        on_delete=models.CASCADE
    )

    def __str__(self):
        return str(self.comment.idcomment)


@python_2_unicode_compatible
class Register(models.Model):
    """
    Model Register
    """
    idregister = models.AutoField(primary_key=True)
    forum = models.ForeignKey(
        Forum, related_name='register_forums', verbose_name=_('Forum'),
        on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='register_users',
        verbose_name=_('User'), on_delete=models.CASCADE
    )
    date = models.DateTimeField(
        _('Date'), blank=True, auto_now=True, db_index=True
    )

    class Meta(object):
        ordering = ['date']
        verbose_name = _('Register')
        verbose_name_plural = _('Registers')

    def __str__(self):
        return str(self.forum) + " " + str(self.user)


@python_2_unicode_compatible
class AbstractProfile(models.Model):
    """
    Model AbstractProfile
    """
    def generate_path_profile(instance, filename):
        """
        Generate path to field photo
        """
        return os.path.join(
            "profiles", "profile_" + str(instance.iduser_id), filename
        )

    idprofile = models.AutoField(primary_key=True, unique=True)
    iduser = models.OneToOneField(
        settings.AUTH_USER_MODEL, related_name="user", db_index=True,
        on_delete=models.CASCADE
    )
    photo = models.FileField(
        upload_to=generate_path_profile, null=True, blank=True,
    )
    about = models.TextField(blank=True, null=True)
    activation_key = models.CharField(max_length=100, null=False, blank=False)
    key_expires = models.DateTimeField(auto_now=False)
    is_troll = models.BooleanField(
        _('Is troll'), default=False,
        help_text=_('If the user is troll')
    )

    def __str__(self):
        return str(self.iduser.username)

    def last_seen(self):
        return cache.get('seen_%s' % self.iduser.username)

    def online(self):
        if self.last_seen():
            now = timezone.now()
            if now > self.last_seen() + timezone.timedelta(
                         seconds=localSettings.USER_ONLINE_TIMEOUT):
                return False
            else:
                return True
        else:
            return False

    class Meta:
        abstract = True


@python_2_unicode_compatible
class Configuration(models.Model):
    """
    Model configuration mussete like logo and class css
    """
    def generate_path_configuration(instance, filename):
        """
        Generate path to field logo
        """
        return os.path.join(
            "configuration", filename
        )

    idconfig = models.AutoField(primary_key=True)
    site = models.OneToOneField(Site)
    logo = models.FileField(
        upload_to=generate_path_configuration, null=True, blank=True,
    )
    logo_width = models.PositiveIntegerField(
        _("Logo width"), null=True, blank=True,
        help_text=_('In pixels')
    )
    logo_height = models.PositiveIntegerField(
        _("Logo height"), null=True, blank=True,
        help_text=_('In pixels')
    )
    custom_css = models.TextField(
        _("Custom design"), null=True, blank=True
    )
    description = models.TextField(_('Description'), blank=True)

    class Meta(object):
        verbose_name = _('Configuration')
        verbose_name_plural = _('Configurations')

    def __str__(self):
        return str(self.idconfig)



from django.conf import settings
from django.conf.urls import include, url
from django.contrib import admin
from django.contrib.auth.decorators import login_required

from musette import views
from musette.feeds import TopicFeed
from musette.api.urls import router


admin.site.site_header = settings.SITE_NAME

urlpatterns = [
    # Url for django-rest-framework
    url(r'^', include(router.urls)),
    url(r'^api/', include('rest_framework.urls', namespace='rest_framework')),
    # Url for django-hitcount
    url(r'hitcount/', include('hitcount.urls', namespace='hitcount')),

    # Url's Django-musette
    url(r'^login/', views.LoginView.as_view(), name='login'),
    url(r'^logout/', views.LogoutView.as_view(), name='logout'),
    url(r'^join/', views.SignUpView.as_view(), name='signup'),
    url(r'^confirm_email/(?P<username>.+)/(?P<activation_key>\w+)',
        views.ConfirmEmailView.as_view(), name='config_email'),
    url(r'^new_key_activation/(?P<username>.+)',
        views.NewKeyActivationView.as_view(), name='new_key_activation'),
    url(r'^reset_password/$', views.reset_password, name='password_reset'),
    url(
        r'^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        views.reset_pass_confirm, name='password_reset_confirm'
    ),
    url(r'^reset/done/$', views.reset_done_pass,
        name='password_reset_complete'),

    url(r'^forums/$', views.ForumsView.as_view(), name='forums'),
    url(r'^forum/(?P<category>.+)/(?P<forum>.+)/$', views.ForumView.as_view(),
        name='forum'),
    url(
        r'^topic/(?P<category>.+)/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/$',
        views.TopicView.as_view(), name='topic'
    ),
    url(
        r'^newtopic/(?P<category>.+)/(?P<forum>.+)/$',
        login_required(views.NewTopicView.as_view()), name='newtopic'
    ),
    url(
        r'^edit_topic/(?P<category>.+)/(?P<forum>.+)/(?P<idtopic>\d+)/$',
        login_required(views.EditTopicView.as_view()), name='edittopic'
    ),
    url(
        r'^delete_topic/$',
        login_required(views.DeleteTopicView.as_view()), name='deletetopic'
    ),
    url(
        r'^open_close_topic/$',
        login_required(views.OpenCloseTopicView.as_view()),
        name="open_close_topic"
    ),
    url(
        r'^like_unlike_topic',
        login_required(views.LikeUnlikeTopicView.as_view()),
        name="like_unlike_topic"
    ),
    url(
        r'^like_unlike_comment',
        login_required(views.LikeUnlikeCommentView.as_view()),
        name="like_unlike_comment"
    ),
    url(
        r'^newcomment/(?P<category>.+)/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/$',
        login_required(views.NewCommentView.as_view()), name='newcomment'
    ),
    url(
        r'^updatecomment/(?P<category>.+)/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/(?P<idcomment>\d+)/$',
        login_required(views.EditCommentView.as_view()), name='updatecomment'
    ),
    url(
        r'^removecomment/(?P<category>.+)/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/(?P<idcomment>\d+)/$',
        login_required(views.DeleteCommentView.as_view()), name='removecomment'
    ),
    url(
        r'^forum_all_notification/$',
        login_required(views.AllNotification.as_view()),
        name='forum_all_notification'
    ),
    url(
        r'^forum_set_notifications/$',
        login_required(views.SetNotifications), name='forum_set_notifications'
    ),
    url(
        r'^new_register/(?P<category>.+)/(?P<forum>.+)/$', login_required(
            views.AddRegisterView.as_view()), name='new_register'
    ),
    url(
        r'^unregister/(?P<category>.+)/(?P<forum>.+)/$', login_required(
            views.UnregisterView.as_view()), name='unregister'
    ),
    url(
        r'^users_forum/(?P<category>.+)/(?P<forum>.+)/$',
        views.UsersForumView.as_view(), name='users_forum'
    ),
    url(
        r'^search_topic/(?P<category>.+)/(?P<forum>.+)/$',
        views.TopicSearch.as_view(), name='search_topic'
    ),
    url(r'^feed/(?P<category>.+)/(?P<forum>.+)/$', TopicFeed(), name='rss'),
    url(
        r'^profile/(?P<username>.+)/$',
        views.ProfileView.as_view(), name='profile'
    ),
    url(
        r'^edit_profile/(?P<username>.+)/$',
        login_required(views.EditProfileView.as_view()), name='edit_profile'
    ),
    url(
        r'profile_is_troll/$',
        login_required(views.IsTrollView.as_view()), name='profile_is_troll'
    ),
]


from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import get_object_or_404

from rest_framework import status, viewsets
from rest_framework.exceptions import PermissionDenied
from rest_framework.permissions import IsAuthenticatedOrReadOnly
from rest_framework.response import Response

from musette import models, realtime, utils
from musette.api import serializers
from musette.api.permissions import ForumPermissions, IsReadOnly


# ViewSets for user
class UserViewSet(viewsets.ReadOnlyModelViewSet):
    User = get_user_model()
    queryset = User.objects.all()
    serializer_class = serializers.UserSerializer
    lookup_field = 'username'


# ViewSets for categiry
class CategoryViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = models.Category.objects.all()
    serializer_class = serializers.CategorySerializer


# ViewSets for forum
class ForumViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = models.Forum.objects.all()
    serializer_class = serializers.ForumSerializer


# ViewSets for topic
class TopicViewSet(viewsets.ModelViewSet):
    queryset = models.Topic.objects.all()
    serializer_class = serializers.TopicSerializer
    permission_classes = (
        IsAuthenticatedOrReadOnly, ForumPermissions,
    )

    def get_permissions(self):
        # If is troll then only is read only
        if self.request.user.is_authenticated():
            if self.request.user.user.is_troll:
                self.permission_classes = [IsReadOnly, ]
        return super(TopicViewSet, self).get_permissions()

    def perform_create(self, serializer):
        request = self.request
        is_my_user = int(request.data['user']) == request.user.id

        # If is my user or is superuser can create
        if is_my_user or request.user.is_superuser:
            forum_id = request.data['forum']
            forum = get_object_or_404(models.Forum, pk=forum_id)
            category = forum.category.name
            # If has permissions
            if utils.user_can_create_topic(category, forum, request.user):
                # Save the record topic
                if serializer.is_valid():
                    # If the forum is moderate send email
                    serializer = utils.check_moderate_topic_email(
                        request, forum, serializer
                    )
                    # Save record
                    topic = serializer.save()
                else:
                    return Response(
                        serializer.errors,
                        status=status.HTTP_400_BAD_REQUEST
                    )

                # Parameters for realtime
                photo = utils.get_photo_profile(request.user.id)
                username = request.user.username
                forum_name = forum.name

                # Get moderators forum and send notification
                list_us = utils.get_moderators_and_send_notification_topic(
                    request, forum, topic
                )

                # Data necessary for realtime
                data = realtime.data_base_realtime(
                    topic, photo, forum_name, username
                )

                # Send new notification realtime
                realtime.new_notification(data, list_us)

                return Response(
                    serializer.data, status=status.HTTP_201_CREATED
                )
            else:
                raise PermissionDenied({
                    "message": "You don't have permission to access"
                })
        else:
            raise PermissionDenied({
                    "message": "Not your user"
                })


# ViewSets for register
class RegisterViewSet(viewsets.ModelViewSet):
    queryset = models.Register.objects.all()
    serializer_class = serializers.RegisterSerializer
    permission_classes = (IsAuthenticatedOrReadOnly, ForumPermissions,)

    def get_permissions(self):
        # If is troll then only is read only
        if self.request.user.is_authenticated():
            if self.request.user.user.is_troll:
                self.permission_classes = [IsReadOnly, ]
        return super(RegisterViewSet, self).get_permissions()

    def create(self, request, **kwargs):
        is_my_user = int(request.data['user']) == request.user.id
        # If is my user or is superuser can create
        if is_my_user or request.user.is_superuser:
            forum_id = request.data['forum']
            exists_register = models.Register.objects.filter(
                forum_id=forum_id, user=request.user
            )

            # If the register not exists
            if exists_register.count() == 0:
                return super(RegisterViewSet, self).create(request, **kwargs)
            else:
                raise PermissionDenied({
                    "message": "You are already Registered"
                })
        else:
            raise PermissionDenied({
                    "message": "Not your user"
                })


# ViewSets for comment
class CommentViewSet(viewsets.ModelViewSet):
    queryset = models.Comment.objects.all()
    serializer_class = serializers.CommentSerializer
    permission_classes = (IsAuthenticatedOrReadOnly, ForumPermissions,)

    def get_permissions(self):
            # If is troll then only is read only
        if self.request.user.is_authenticated():
            if self.request.user.user.is_troll:
                self.permission_classes = [IsReadOnly, ]
        return super(CommentViewSet, self).get_permissions()

    def perform_create(self, serializer):
        request = self.request
        is_my_user = int(request.data['user']) == request.user.id
        # If is my user or is superuser can create
        if is_my_user or request.user.is_superuser:
            # Save the record comment
            if serializer.is_valid():
                comment = serializer.save()
            else:
                return Response(
                    serializer.errors,
                    status=status.HTTP_400_BAD_REQUEST
                )

            topic_id = request.data['topic']
            topic = get_object_or_404(models.Topic, pk=topic_id)

            # Parameters for notification comments
            photo = utils.get_photo_profile(request.user.id)
            username = request.user.username
            forum = topic.forum.name

            # Send notifications comment
            params = utils.get_users_and_send_notification_comment(
                request, topic, comment
            )
            list_us = params['list_us']
            list_email = params['list_email']

            # Get url for email
            url = reverse_lazy('topic', kwargs={
                'category': topic.forum.category, 'forum': forum,
                'slug': topic.slug, 'idtopic': str(topic.idtopic)
            })

            # Send e    mail
            utils.send_mail_comment(str(url), list_email)

            # Data necessary for realtime
            data = realtime.data_base_realtime(topic, photo, forum, username)

            # Send new notification realtime
            realtime.new_notification(data, list_us)

            # Send new comment in realtime
            comment_description = request.data['description']
            realtime.new_comment(data, comment_description)

            return Response(
                serializer.data, status=status.HTTP_201_CREATED
            )
        else:
            raise PermissionDenied({
                    "message": "Not your user"
                })


# ViewSets for profile
class ProfileViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = utils.get_main_model_profile().objects.all()
    serializer_class = serializers.ProfileSerializer



from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _


def valid_extension(value):
    '''
    Function that valid extension
    when upload file in form
    '''
    if (not value.name.endswith('.png') and
            not value.name.endswith('.jpeg') and
            not value.name.endswith('.gif') and
            not value.name.endswith('.bmp') and
            not value.name.endswith('.jpg')):

        text = _("Files allowed")
        files = ".jpg, .jpeg, .png, .gif, .bmp"
        raise ValidationError(text + ': ' + files)


import base64
import hashlib
import os
import random
import shutil

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from musette.models import (
    Forum, Topic, Comment, Register,
    Notification, AbstractProfile
)
from musette.email import send_mail


def exists_folder(route):
    """
    This method verify that exists
    folder in base to route
    """
    if os.path.exists(route):
        return True
    else:
        return False


def remove_folder(route_folder):
    """
    This method remove one folder
    """
    try:
        shutil.rmtree(route_folder)
    except Exception:
        pass


def remove_file(route_file):
    """
    This method remove one file
    in base to route and image
    """
    if route_file != "" and not route_file is None:
        if os.path.exists(route_file):
            os.remove(route_file)


def get_folder_attachment(topic):
    """
    This method return the path of one
    folder attachment for app forum
    """
    folder = ""
    folder = "forum_" + str(topic.forum_id)
    folder = folder + "_user_" + str(topic.user.username)
    folder = folder + "_topic_" + str(topic.id_attachment)
    path_folder = os.path.join("forum", folder)
    media_path = settings.MEDIA_ROOT
    path = media_path + "/" + path_folder

    return path


def remove_folder_attachment(idtopic):
    """
    This method remove folder attachment
    and subtract one topic.
    """
    # Subtract one topic
    topic = get_object_or_404(Topic, idtopic=idtopic)
    forum = get_object_or_404(
        Forum, category__name=topic.forum.category.name,
        name=topic.forum, hidden=False
    )
    tot = forum.topics_count
    tot = tot - 1
    Forum.objects.filter(
        category__name=topic.forum.category.name,
        name=topic.forum, hidden=False
    ).update(
        topics_count=tot
    )

    path = get_folder_attachment(topic)

    # Remove attachment if exists
    if exists_folder(path):
        remove_folder(path)


def get_main_model_profile():
    """
    This method return the model profile defined by user
    """
    subclasses = AbstractProfile.__subclasses__()
    if len(subclasses) > 0:
        try:
            return subclasses[0]
        except Exception:
            raise BaseException("Occurs one error to get model profile")
    else:
        raise BaseException("It is not defined profile model")


def get_app_model(instance):
    """
    This method get app_label from model instance
    """
    return instance._meta.app_label


def get_count_fields_model(instance):
    """
    This method get count fields from model instance
    """
    return len(instance._meta.fields)


def get_id_profile(iduser):
    """
    This method return one id of model profile
    """
    ModelProfile = get_main_model_profile()
    profile = get_object_or_404(ModelProfile, iduser=iduser)

    return profile


def get_users_topic(topic, myuser):
    """
    This method return all users of one topic, else my user
    """
    comments = Comment.objects.filter(topic_id=topic.idtopic)
    lista_us = []
    for comment in comments:
        if comment.user_id != myuser:
            if not (comment.user_id in lista_us):
                lista_us.append(comment.user_id)

    return lista_us


def get_notifications(iduser):
    """
    This method return Notification of one user
    """
    try:
        notif = Notification.objects.filter(
            iduser=iduser).order_by("-date")
    except Notification.DoesNotExist:
        notif = None

    return notif


def get_datetime_topic(date):
    """
    This method return info one datetime for topic or notification
    """
    flag = True
    now = timezone.now()
    # If is beteween 1 and 10 return days
    difference = (now - date).days

    # If is this days return hours
    if difference == 0:
        flag = False
        diff = now - date
        minutes = (diff.seconds // 60) % 60
        hours = diff.seconds // 3600
        if minutes < 60 and hours == 0:
            difference = "%s %s" % (
                _("ago"), str((diff.seconds // 60) % 60) + "m ")
        else:
            difference = "%s %s" % (_("ago"), str(diff.seconds // 3600) + "h ")

    # If is days
    if flag:
        difference = "%s %s %s" % (_("ago"), str(difference), _("days ago"))

    return difference


def basename(value):
    """
    This method return basename of one path
    """
    return os.path.basename(value)


def get_route_file(file_path, file_name):
    """
    This method build the path for a file MEDIA
    """
    try:
        route_file = file_path + "/" + file_name
    except Exception:
        route_file = ""

    return route_file


def get_photo_profile(iduser):
    """
    This method return photo profile
    """
    default_photo = static("musette/img/profile.png")
    ModelProfile = get_main_model_profile()
    profile = ModelProfile.objects.filter(iduser=iduser)
    if profile.count() > 0:
        photo = profile[0].photo
        if photo:
            field_photo = settings.MEDIA_URL + str(photo)
        else:
            field_photo = default_photo
    else:
        field_photo = default_photo
    return field_photo


def send_welcome_email(email, username, activation_key):
    """
    This method send email for confirm user
    """
    username = base64.b64encode(username.encode("utf-8")).decode("ascii")
    content = _(
        "Thank you for joining to %(site)s "
        "please enter to confirm your email to this address:"
    ) % {
        'site': settings.SITE_NAME
    }
    urlContent = "confirm_email/" + username + "/" + activation_key
    send_mail(
        _("Welcome to " + settings.SITE_NAME),
        _(content) + settings.SITE_URL + urlContent,
        settings.EMAIL_MUSETTE,
        [email],
        fail_silently=False
    )


def send_mail_comment(url, list_email):
    """
    Send email comment
    """
    if settings.SITE_URL.endswith("/"):
        site = settings.SITE_URL[:-1]
    else:
        site = settings.SITE_URL

    title_email = _("New comment in %(site)s") % {
        'site': settings.SITE_NAME
    }

    message = _("You have one new comment in the topic: %(site)s") % {
        'site': site + url
    }

    email_from = settings.EMAIL_MUSETTE
    if email_from:
        send_mail(
            title_email, message, email_from,
            list_email, fail_silently=False
        )


def send_mail_topic(email_moderator, forum):
    """
    Send email topic
    """
    # Send email to moderator
    if settings.SITE_URL.endswith("/"):
        site = settings.SITE_URL + "forum/" + forum.name
    else:
        site = settings.SITE_URL + "/forum/" + forum.name

    site_name = settings.SITE_NAME
    title_email = _("New topic in %(site)s ") % {'site': site_name}
    message = _("You have one new topic to moderate: %(site)s") % {
        'site': site
    }
    email_from = settings.EMAIL_MUSETTE

    if email_from:
        send_mail(
            title_email, message, email_from,
            [email_moderator], fail_silently=False
        )


def get_data_confirm_email(email):
    """
    This method return info for email confir
    """
    salt = hashlib.sha1(str(random.random()).encode("utf-8")).hexdigest()[:5]
    key = salt.encode("utf-8") + email.encode("utf-8")
    activation_key = hashlib.sha1(key).hexdigest()
    key_expires = timezone.now() + timezone.timedelta(2)

    return {
        'activation_key': activation_key,
        'key_expires': key_expires
    }


def is_user_moderator_forum(category, forum, user):
    """
    Check if user is moderator forum
    """
    forum = get_object_or_404(Forum, category__name=category, name=forum)
    if user in forum.moderators.all():
        return True
    else:
        return False


def user_can_create_topic(category, forum, user):
    """
    Check if user can create topic
    """
    is_moderator = is_user_moderator_forum(category, forum, user)
    is_register = Register.objects.filter(forum=forum, user=user).count()
    # If is superuser or moderator or is register in the forum
    if user.is_superuser or is_moderator or is_register > 0:
        return True
    else:
        return False


def get_total_forum_moderate_user(user):
    """
    Get total of forums that moderate one user
    """
    return Forum.objects.filter(
        moderators=user
    ).count()


def save_notification_model(related_object, idobject, iduser, is_topic):
    """
    Save new notificaiton
    """
    if is_topic:
        is_comment = False
    else:
        is_comment = True

    now = timezone.now()
    notification = Notification(
        iduser=iduser, is_view=False,
        idobject=idobject, date=now,
        is_topic=is_topic, is_comment=is_comment,
        content_type=related_object
    )
    notification.save()


def get_moderators_and_send_notification_topic(request, forum, topic):
    """
    Get list moderators to send notification for realtime
    and send notificaiton to model Notification for topic
    """
    # Get moderators forum
    list_us = []

    related_object = ContentType.objects.get_for_model(topic)
    for moderator in forum.moderators.all():
        # If not is my user
        if moderator.id != request.user.id:
            # Send notification to moderator
            save_notification_model(
                related_object, topic.idtopic, moderator.id, True
            )
            list_us.append(moderator.id)

    return list_us


def get_users_and_send_notification_comment(request, topic, comment):
    """
    Get list users to send notification for realtime
    and send notificaiton to model Notification for comment
    """
    now = timezone.now()

    myuser = request.user.id
    # Send notifications
    list_us = get_users_topic(topic, myuser)
    list_email = []

    # If not exists user that create topic, add
    user_original_topic = topic.user.id
    user_email = topic.user.email
    comment_user = comment.user.id

    # If the notificacion is mine send to all but not to me
    if user_original_topic == myuser and comment_user == myuser:
        # Not make nothing but list_user is already
        # Not send email
        pass
    # If the notificacion is mine send to all but not to create to comment
    elif user_original_topic == myuser and comment_user != myuser:
        # The user comment not exists in list_us
        # Add user that created topic
        list_us.append(user_original_topic)
        # Add user for send email
        list_email.append(user_email)
    # If the notificacion not is mine send to all but not to me
    elif user_original_topic != myuser and comment_user == myuser:
        # Check if exists the created topic
        if not(user_original_topic in list_us):
            # Send to created topic
            list_us.append(user_original_topic)

        # Add user for send email to created topic
        list_email.append(user_email)
    # If the notificacion not is mine send to all but not to create to comment
    elif user_original_topic != myuser and comment_user != myuser:
        # Check if exists the created topic
        if not(user_original_topic in list_us):
            # Send to created topic
            list_us.append(user_original_topic)

        # Add user for send email to created topic
        list_email.append(user_email)

    # Get content type for comment model
    related_object_type = ContentType.objects.get_for_model(comment)
    for user in list_us:
        save_notification_model(
            related_object_type, comment.idcomment, user, False
        )

    return {
        'list_us': list_us,
        'list_email': list_email
    }


def check_moderate_topic_email(request, forum, obj):
    """
    Check if moderate topic and is moderate
    send email to moderators
    """
    # If the forum is moderate
    if forum.is_moderate:
        # If is moderator, so the topic is moderate
        if request.user in forum.moderators.all():
            obj.moderate = True
        elif request.user.is_superuser:
            obj.moderate = True
        else:
            obj.moderate = False

            # Get moderators forum
            for moderator in forum.moderators.all():
                # Send email
                send_mail_topic(moderator.email, forum)
    else:
        obj.moderate = True

    return obj



from django.db import router
from django.contrib import admin, messages
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from django.contrib.admin import helpers
from django.contrib.admin.utils import get_deleted_objects
from django.core.exceptions import PermissionDenied
from django.template.response import TemplateResponse
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from musette import forms, models, utils


class TopicAdmin(admin.ModelAdmin):
    form = forms.FormAdminTopic
    list_display = ('title', 'forum', 'date', 'moderate', 'is_close')
    list_filter = ['title', 'date', 'moderate', 'is_close']
    search_fields = ['title', 'date', 'moderate', 'is_close']
    actions = ['delete_topic']

    def get_form(self, request, obj=None, **kwargs):
        ModelForm = super(TopicAdmin, self).get_form(request, obj, **kwargs)

        class ModelFormMetaClass(ModelForm):
            def __new__(cls, *args, **kwargs):
                kwargs['request'] = request
                return ModelForm(*args, **kwargs)

        return ModelFormMetaClass

    def get_queryset(self, request):
        qs = super(TopicAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs

        forums = models.Forum.objects.filter(
            moderators=request.user.id
        )
        lista = []
        for f in forums:
            lista.append(f.idforum)

        return qs.filter(forum_id__in=lista)

    def get_actions(self, request):
        actions = super(TopicAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def delete_topic(self, request, queryset):
        """
        This method remove topic's selected in the admin django.
        Can remove one o more records.
        """
        if not self.has_delete_permission(request):
            raise PermissionDenied

        if request.POST.get("post"):
            for obj in queryset:
                idtopic = obj.idtopic
                # Remove folder attachment
                utils.remove_folder_attachment(idtopic)
                # Delete record
                models.Topic.objects.filter(
                    idtopic=idtopic
                ).delete()

            n = queryset.count()
            self.message_user(
                request, _("Successfully deleted %(count)d record/s.") % {
                    "count": n, }, messages.SUCCESS
            )

            return None
        else:

            opts = self.model._meta

            if len(queryset) == 1:
                objects_name = force_text(opts.verbose_name)
            else:
                objects_name = force_text(opts.verbose_name_plural)

            using = router.db_for_write(self.model)

            del_obj, model_c, perms_n, protected = get_deleted_objects(
                queryset, opts, request.user, self.admin_site, using
            )

            context = {
                'title': "",
                'delete_topic': [queryset],
                'ids': queryset.values_list("idtopic"),
                'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
                'opts': opts,
                'objects_name': objects_name,
                'deletable_objects': [del_obj],
                'action': 'delete_topic',
            }

            return TemplateResponse(
                request, 'musette/admin/confirm_delete.html', context
            )

    delete_topic.short_description = _(
        "Delete selected %(verbose_name_plural)s"
    )


class ForumAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'category', 'forum_description',
        'topics_count', 'is_moderate', 'get_moderators'
    )
    list_filter = ['name', 'category']
    search_fields = ['name']
    actions = ['delete_forum']

    def get_moderators(self, obj):
        return "\n".join([p.username for p in obj.moderators.all()])
    get_moderators.short_description = _("Moderators")

    def get_actions(self, request):
        actions = super(ForumAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def delete_forum(self, request, queryset):
        """
        This method remove forum selected
        in the admin django. Can remove one
        o more records.
        """
        if not self.has_delete_permission(request):
            raise PermissionDenied

        if request.POST.get("post"):
            for obj in queryset:
                idforum = obj.idforum
                # Remove permissions to moderators
                obj.remove_user_permissions_moderator()

                # Delete record
                models.Forum.objects.filter(
                    idforum=idforum
                ).delete()

            n = queryset.count()
            self.message_user(
                request, _("Successfully deleted %(count)d record/s.") % {
                    "count": n, }, messages.SUCCESS
            )

            return None
        else:

            opts = self.model._meta

            if len(queryset) == 1:
                objects_name = force_text(opts.verbose_name)
            else:
                objects_name = force_text(opts.verbose_name_plural)

            using = router.db_for_write(self.model)

            del_obj, model_c, perms_n, protected = get_deleted_objects(
                queryset, opts, request.user, self.admin_site, using
            )

            context = {
                'title': "",
                'delete_topic': [queryset],
                'ids': queryset.values_list("idforum"),
                'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
                'opts': opts,
                'objects_name': objects_name,
                'deletable_objects': [del_obj],
                'action': 'delete_forum'
            }

            return TemplateResponse(
                request, 'musette/admin/confirm_delete.html', context
            )

    delete_forum.short_description = _(
        "Delete selected %(verbose_name_plural)s"
    )


class CommentAdmin(admin.ModelAdmin):
    list_display = (
        'topic', 'forum', 'user',
    )
    list_filter = ['topic', 'user']
    search_fields = ['topic', 'user']

    def forum(self, obj):
        return obj.topic.forum

    forum.short_description = _("Forum")
    forum.admin_order_field = 'topic__forum'


class ConfigurationAdmin(admin.ModelAdmin):
    list_display = ('site', 'logo',)
    form = forms.FormAdminConfiguration


class MessageForumAdmin(admin.ModelAdmin):
    list_display = (
        'forum', 'message_information',
        'message_expires_from', 'message_expires_to'
    )


class RegisterAdmin(admin.ModelAdmin):
    list_display = (
        'forum', 'user',
    )


class ProfileInline(admin.StackedInline):
    model = utils.get_main_model_profile()
    can_delete = False
    verbose_name_plural = _('Profile')
    fk_name = 'iduser'
    form = forms.FormAdminProfile


class CustomUserAdmin(UserAdmin):
    inlines = (ProfileInline, )

    def get_inline_instances(self, request, obj=None):
        if not obj:
            return list()
        return super(CustomUserAdmin, self).get_inline_instances(request, obj)


User = get_user_model()

try:
    admin.site.unregister(User)
except Exception:
    pass

admin.site.register(User, CustomUserAdmin)
admin.site.register(models.Category)
admin.site.register(models.Register, RegisterAdmin)
admin.site.register(models.Forum, ForumAdmin)
admin.site.register(models.Topic, TopicAdmin)
admin.site.register(models.Comment, CommentAdmin)
admin.site.register(models.Configuration, ConfigurationAdmin)
admin.site.register(models.MessageForum, MessageForumAdmin)



from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand

from musette.utils import get_main_model_profile, get_data_confirm_email


class Command(BaseCommand):
    help = "Create record profile for SuperAdmin users."

    def handle(self, *args, **options):
        # Get model profile
        Profile = get_main_model_profile()
        # Get users super-admin
        User = get_user_model()
        users = User.objects.filter(is_superuser=True)

        # Create recrod profile
        if users.count() > 0:
            for user in users:
                if not Profile.objects.filter(iduser=user).exists():
                    data = get_data_confirm_email(user.email)
                    Profile.objects.create(
                        iduser=user, photo="", about="",
                        activation_key=data['activation_key'],
                        key_expires=data['key_expires']
                    )
                    self.stdout.write('Profile created: ' + user.username)
                else:
                    self.stdout.write('Profile ' + user.username + ' exists.')
            self.stdout.write("Finished.")
        else:
            self.stdout.write("There is no super-user registered.")



import threading

from django.core.mail import send_mail as core_send_mail
from django.core.mail import EmailMultiAlternatives


class EmailThread(threading.Thread):
    """
    Class for send email async with thread
    """
    def __init__(self, subject, body, from_email, recipient_list,
                 fail_silently, html):
        self.subject = subject
        self.body = body
        self.recipient_list = recipient_list
        self.from_email = from_email
        self.fail_silently = fail_silently
        self.html = html
        threading.Thread.__init__(self)

    def run(self):
        msg = EmailMultiAlternatives(
            self.subject, self.body, self.from_email, self.recipient_list
        )
        if self.html:
            msg.attach_alternative(self.html, "text/html")
        msg.send(self.fail_silently)


def send_mail(subject, body, from_email, recipient_list, fail_silently=False,
              html=None, *args, **kwargs):
    """
    Send email async
    """
    EmailThread(
        subject, body, from_email, recipient_list, 
        fail_silently, html
    ).start()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-01-24 18:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('musette', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='forum',
            name='name',
            field=models.CharField(max_length=80, verbose_name='Name'),
        ),
    ]



from django.http import HttpResponseRedirect


def user_is_troll(f):
    """
    Decorator for check if the user is a troll
    """
    def wrap(request, *args, **kwargs):
        if request.user.is_authenticated():
            is_troll = request.user.user.is_troll
            if not is_troll:
                return f(request, *args, **kwargs)
            else:
                return HttpResponseRedirect('/')
        else:
            return HttpResponseRedirect('/')
    return wrap



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-01-24 18:01
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import musette.models
import musette.validators


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0002_alter_domain_unique'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('idcategory', models.AutoField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=80)),
                ('position', models.IntegerField(blank=True, default=0)),
                ('hidden', models.BooleanField(default=False, help_text='If checked, this category will be visible only for staff')),
            ],
            options={
                'verbose_name_plural': 'Categories',
                'ordering': ['position'],
                'verbose_name': 'Category',
            },
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('idcomment', models.AutoField(primary_key=True, serialize=False)),
                ('date', models.DateTimeField(auto_now=True, db_index=True, verbose_name='Date')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
            ],
            options={
                'verbose_name_plural': 'Comments',
                'ordering': ['date'],
                'verbose_name': 'Comment',
            },
        ),
        migrations.CreateModel(
            name='Configuration',
            fields=[
                ('idconfig', models.AutoField(primary_key=True, serialize=False)),
                ('logo', models.FileField(blank=True, null=True, upload_to=musette.models.Configuration.generate_path_configuration)),
                ('logo_width', models.PositiveIntegerField(blank=True, help_text='In pixels', null=True, verbose_name='Logo width')),
                ('logo_height', models.PositiveIntegerField(blank=True, help_text='In pixels', null=True, verbose_name='Logo height')),
                ('custom_css', models.TextField(blank=True, null=True, verbose_name='Custom design')),
                ('site', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='sites.Site')),
            ],
            options={
                'verbose_name_plural': 'Configurations',
                'verbose_name': 'Configuration',
            },
        ),
        migrations.CreateModel(
            name='Forum',
            fields=[
                ('idforum', models.AutoField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=80, unique=True, verbose_name='Name')),
                ('position', models.IntegerField(blank=True, default=0, verbose_name='Position')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
                ('date', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Date')),
                ('topics_count', models.IntegerField(blank=True, default=0, editable=False, verbose_name='Topics count')),
                ('hidden', models.BooleanField(default=False, help_text='If hide the forum in the index page', verbose_name='Hidden')),
                ('is_moderate', models.BooleanField(default=False, help_text='If the forum is moderated', verbose_name='Check topics')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='categories', to='musette.Category', verbose_name='Category')),
                ('moderators', models.ManyToManyField(related_name='moderators', to=settings.AUTH_USER_MODEL, verbose_name='Moderators')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='parents', to='musette.Forum', verbose_name='Parent forum')),
            ],
            options={
                'verbose_name_plural': 'Forums',
                'ordering': ['category', 'position', 'name'],
                'verbose_name': 'Forum',
            },
        ),
        migrations.CreateModel(
            name='MessageForum',
            fields=[
                ('idmessage_forum', models.AutoField(primary_key=True, serialize=False)),
                ('message_information', models.TextField(help_text='If you want to report a message to a forum', verbose_name='Message to inform')),
                ('message_expires_from', models.DateTimeField(help_text='Date from message expired', verbose_name='Message expires from')),
                ('message_expires_to', models.DateTimeField(help_text='Date to message expired', verbose_name='Message expires to')),
                ('forum', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='message_information', to='musette.Forum', verbose_name='Forum')),
            ],
            options={
                'verbose_name_plural': 'Messages for forums',
                'verbose_name': 'Message for forums',
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('idnotification', models.AutoField(primary_key=True, serialize=False)),
                ('idobject', models.PositiveIntegerField(null=True)),
                ('iduser', models.IntegerField(default=0)),
                ('is_topic', models.BooleanField(default=0)),
                ('is_comment', models.BooleanField(default=0)),
                ('is_view', models.BooleanField(default=0)),
                ('date', models.DateTimeField(blank=True, db_index=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ['date'],
            },
        ),
        migrations.CreateModel(
            name='Register',
            fields=[
                ('idregister', models.AutoField(primary_key=True, serialize=False)),
                ('date', models.DateTimeField(auto_now=True, db_index=True, verbose_name='Date')),
                ('forum', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='register_forums', to='musette.Forum', verbose_name='Forum')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='register_users', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name_plural': 'Registers',
                'ordering': ['date'],
                'verbose_name': 'Register',
            },
        ),
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('idtopic', models.AutoField(primary_key=True, serialize=False)),
                ('slug', models.SlugField(max_length=100)),
                ('title', models.CharField(max_length=80, verbose_name='Title')),
                ('date', models.DateTimeField(auto_now=True, verbose_name='Date')),
                ('last_activity', models.DateTimeField(auto_now=True, verbose_name='Last activity')),
                ('description', models.TextField(verbose_name='Description')),
                ('id_attachment', models.CharField(blank=True, max_length=200, null=True)),
                ('attachment', models.FileField(blank=True, null=True, upload_to=musette.models.Topic.generate_path, validators=[musette.validators.valid_extension], verbose_name='File')),
                ('is_close', models.BooleanField(default=False, help_text='If the topic is close', verbose_name='Closed topic')),
                ('moderate', models.BooleanField(default=False, help_text='If the topic is moderated', verbose_name='Moderate')),
                ('is_top', models.BooleanField(default=False, help_text='If the topic is important and it will go top', verbose_name='Top')),
                ('forum', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='forums', to='musette.Forum', verbose_name='Forum')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='users', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name_plural': 'Topics',
                'ordering': ['forum', 'last_activity', 'title', 'date'],
                'verbose_name': 'Topic',
            },
        ),
        migrations.AddField(
            model_name='comment',
            name='topic',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='topics', to='musette.Topic', verbose_name='Topic'),
        ),
        migrations.AddField(
            model_name='comment',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comment_users', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



# encoding:utf-8
from django import template

from ..utils import get_photo_profile

register = template.Library()


@register.filter
def get_photo(user):
    """
    This tag return the path photo profile
    """
    field_photo = get_photo_profile(user)
    return field_photo



import base64
from itertools import chain

from django.db.models import F, Q
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import get_user_model, login, logout
from django.contrib.auth.forms import PasswordResetForm, SetPasswordForm
from django.contrib.auth.tokens import default_token_generator
from django.contrib.auth.views import (
    password_reset, password_reset_complete,
    password_reset_confirm
)
from django.core.urlresolvers import reverse_lazy
from django.http import Http404, HttpResponse, HttpResponseRedirect, QueryDict
from django.shortcuts import render, get_object_or_404, redirect
from django.template import defaultfilters
from django.views.generic import View
from django.views.generic.edit import FormView
from django.utils.crypto import get_random_string
from django.utils import timezone
from django.utils.html import conditional_escape
from django.utils.translation import ugettext_lazy as _

from musette import forms, mixins, models, realtime, utils


class LoginView(FormView):
    """
    Login View
    """
    template_name = "musette/login.html"
    form_class = forms.FormLogin
    success_url = reverse_lazy("forums")

    def get(self, request, *args, **kwargs):
        # Check if is logged, if is trur redirect to home
        if request.user.is_authenticated():
            return ForumsView.as_view()(request)
        else:
            # No is logged, redirecto index login
            data = {
                'form': self.form_class
            }
            return render(request, self.template_name, data)

    def post(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        form = self.get_form(form_class)

        # Check if is authenticated and form valid
        if not request.user.is_authenticated():
            if form.is_valid():
                # This method is one method of class
                # FormLogin in forms.py and is the
                # responsible of authenticate to user
                user = form.form_authenticate()
                if user:
                    if user.is_active:
                        # Login is correct
                        login(request, user)
                        return redirect("forums")
                    else:
                        messages.error(request, _("The user is not active"))
                        return self.form_invalid(form, **kwargs)
                else:
                    return self.form_invalid(form, **kwargs)
            else:
                return self.form_invalid(form, **kwargs)
        else:
            return redirect("forums")


class LogoutView(View):
    """
    View logout
    """
    def get(self, request, *args, **kwargs):
        if request.user.is_authenticated():
            logout(request)

        return redirect("forums")


class SignUpView(FormView):
    """
    This view is responsible of
    create one new user
    """
    template_name = "musette/signup.html"
    form_class = forms.FormSignUp
    success_url = reverse_lazy("signup")

    def get(self, request, *args, **kwargs):
        if request.user.is_authenticated():
            return redirect("forums")
        else:
            data = {'form': self.form_class}
            return render(request, self.template_name, data)

    def post(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        form = self.get_form(form_class)

        if not request.user.is_authenticated():
            if form.is_valid():
                form.create_user()
                msj = _(
                    "Registration was successful. Please, check your email "
                    "to validate the account."
                )
                messages.success(request, msj)
                return self.form_valid(form, **kwargs)
            else:
                messages.error(request, _("Invalid form"))
                return self.form_invalid(form, **kwargs)
        else:
            return redirect("forums")


class ConfirmEmailView(View):
    """
    Form confirm email
    """
    template_name = "musette/confirm_email.html"

    def get(self, request, username, activation_key, *args, **kwargs):
        if request.user.is_authenticated():
            return redirect("forums")

        # Decoding username
        username = base64.b64decode(username.encode("utf-8")).decode("ascii")
        # Parameters for template
        data = {'username': username}

        # Get model profile
        ModelProfile = utils.get_main_model_profile()

        # Check if not expired key
        user_profile = get_object_or_404(
            ModelProfile, activation_key=activation_key
        )

        if user_profile.key_expires < timezone.now():
            return render(request, "musette/confirm_email_expired.html", data)

        # Active user
        User = get_user_model()
        user = get_object_or_404(User, username=username)
        user.is_active = True
        user.save()
        return render(request, self.template_name, data)


class NewKeyActivationView(View):
    """
    View for get a new key activation
    """
    template_name = "musette/confirm_email_expired.html"

    def post(self, request, username, *args, **kwargs):
        if request.user.is_authenticated():
            return redirect("forums")

        User = get_user_model()
        user = get_object_or_404(User, username=username)
        email = user.email

        # For confirm email
        data = utils.get_data_confirm_email(email)

        # Get model profile
        ModelProfile = utils.get_main_model_profile()

        # Update activation key
        profile = get_object_or_404(ModelProfile, iduser=user)
        profile.activation_key = data['activation_key']
        profile.key_expires = data['key_expires']
        profile.save()

        # Send email for confirm user
        utils.send_welcome_email(email, username, data['activation_key'])
        data = {'username': username, 'new_key': True}
        return render(request, self.template_name, data)


def reset_password(request):
    """
    This view contains the form
    for reset password of user
    """
    if request.user.is_authenticated():
        return redirect("forums")

    if request.method == "POST":
        messages.success(request, _('Please, check your email'))

    return password_reset(
        request,
        template_name='musette/password_reset_form.html',
        email_template_name='musette/password_reset_email.html',
        subject_template_name='musette/password_reset_subject.txt',
        password_reset_form=PasswordResetForm,
        token_generator=default_token_generator,
        post_reset_redirect='password_reset',
        from_email=None,
        extra_context=None,
        html_email_template_name=None
    )


def reset_pass_confirm(request, uidb64, token):
    """
    This view display form reset confirm pass
    """
    if request.user.is_authenticated():
        return redirect("forums")

    return password_reset_confirm(
        request, uidb64=uidb64, token=token,
        template_name='musette/password_reset_confirm.html',
        token_generator=default_token_generator,
        set_password_form=SetPasswordForm,
        post_reset_redirect=None,
        extra_context=None
    )


def reset_done_pass(request):
    """
    This view display messages
    that successful reset pass
    """
    if request.user.is_authenticated():
        return redirect("forums")

    return password_reset_complete(
        request, extra_context=None,
        template_name='musette/password_reset_complete.html',
    )


class ForumsView(View):
    """
    This view display all forum registered
    """
    template_name = "musette/index.html"

    def get(self, request, *args, **kwargs):
        # Get categories that not hidden
        categories = models.Category.objects.filter(hidden=False)

        data = {
            'categories': categories
        }

        return render(request, self.template_name, data)


class ForumView(View):
    """
    This view display one forum registered
    """
    def get(self, request, category, forum, *args, **kwargs):

        template_name = "musette/forum_index.html"
        page_template = "musette/forum.html"

        # Get topics forum
        forum = get_object_or_404(
            models.Forum, category__name=category, name=forum, hidden=False
        )
        topics = models.Topic.objects.filter(
            forum_id=forum.idforum
        ).order_by("-is_top", "-last_activity", "-date")

        # Get forum childs
        forums_childs = models.Forum.objects.filter(parent=forum, hidden=False)

        iduser = request.user.id
        if iduser:
            try:
                models.Register.objects.get(
                    forum_id=forum.idforum, user_id=iduser
                )
                register = True
            except models.Register.DoesNotExist:
                register = False

        else:
            register = False

        # Get messages for forum
        now = timezone.now()
        message_queryset = forum.message_information.filter(
            Q(message_expires_from__lte=now, message_expires_to__gte=now),
            Q(message_expires_to__hour__lte=now.hour)
        )

        # Get if has message and if is correct, get message
        if message_queryset.count() > 0:
            has_message_forum = True
            message_forum = message_queryset[0].message_information
        else:
            has_message_forum = False
            message_forum = ""

        data = {
            'forum': forum,
            'forums_childs': forums_childs,
            'topics': topics,
            'register': register,
            'has_message_forum': has_message_forum,
            'message_forum': message_forum
        }

        if request.is_ajax():
            template_name = page_template

        return render(request, template_name, data)


class TopicView(View):
    """
    This view display one Topic of forum
    """
    def get(self, request, category, forum, slug, idtopic, *args, **kwargs):

        template_name = "musette/topic_index.html"
        page_template = "musette/topic.html"

        # Get topic
        forum = get_object_or_404(
            models.Forum, category__name=category, name=forum, hidden=False
        )
        topic = get_object_or_404(models.Topic, idtopic=idtopic, slug=slug)

        # Form for comments
        form_comment = forms.FormAddComment()

        # Get comments of the topic
        comments = models.Comment.objects.filter(topic_id=idtopic)

        # Get photo of created user topic
        photo = utils.get_photo_profile(topic.user.id)

        # Get suggest topic
        words = topic.title.split(" ")
        condition = Q()
        for word in words:
            condition |= Q(title__icontains=word)

        suggest = models.Topic.objects.filter(condition).exclude(
            idtopic=topic.idtopic
        )[:10]

        data = {
            'topic': topic,
            'form_comment': form_comment,
            'comments': comments,
            'photo': photo,
            'suggest': suggest
        }

        if request.is_ajax():
            template_name = page_template
        return render(request, template_name, data)


class NewTopicView(mixins.UserTrollMixin, FormView):
    """
    This view allowed add new topic
    """
    template_name = "musette/new_topic.html"
    form_class = forms.FormAddTopic

    def get_success_url(self):
        return reverse_lazy(
            'forum', kwargs={
                'category': self.kwargs['category'],
                'forum': self.kwargs['forum']
            }
        )

    def get_context_data(self, **kwargs):
        context = super(NewTopicView, self).get_context_data(**kwargs)
        context['forum'] = self.kwargs['forum']
        context['category'] = self.kwargs['category']
        return context

    def get(self, request, category, forum, *args, **kwargs):

        data = {
            'form': self.form_class,
            'forum': forum,
            'category': category,
        }
        return render(request, self.template_name, data)

    def post(self, request, category, forum, *args, **kwargs):
        # Form new topic
        form = forms.FormAddTopic(request.POST, request.FILES)

        if form.is_valid():
            obj = form.save(commit=False)

            now = timezone.now()
            User = get_user_model()
            user = User.objects.get(id=request.user.id)
            # Get forum
            forum = get_object_or_404(
                models.Forum, category__name=category, name=forum
            )
            title = conditional_escape(request.POST['title'])

            obj.date = now
            obj.user = user
            obj.forum = forum
            obj.title = title
            obj.slug = defaultfilters.slugify(request.POST['title'])

            # If has attachment
            if 'attachment' in request.FILES:
                id_attachment = get_random_string(length=32)
                obj.id_attachment = id_attachment

                file_name = request.FILES['attachment']
                obj.attachment = file_name

            # If the forum is moderate send email
            obj = utils.check_moderate_topic_email(request, forum, obj)
            # Save topic
            obj.save()

            # Get moderators forum and send notification
            list_us = utils.get_moderators_and_send_notification_topic(
                request, forum, obj
            )

            # Get photo profile
            username = request.user.username
            photo = utils.get_photo_profile(request.user.id)

            # Data necessary for realtime
            data = realtime.data_base_realtime(
                obj, photo, forum.name, username
            )

            # Add to real time new notification
            realtime.new_notification(data, list_us)

            # Message for user. Form success
            messages.success(
                request, _("The topic '%(topic)s' was successfully created")
                % {'topic': obj.title}
            )
            return self.form_valid(form, **kwargs)
        else:
            messages.error(request, _("Invalid form"))
            return self.form_invalid(form, **kwargs)


class EditTopicView(mixins.UserTrollMixin, FormView):
    """
    This view allowed edit topic
    """
    template_name = "musette/edit_topic.html"
    form_class = forms.FormEditTopic

    def get_success_url(self):
        return reverse_lazy(
            'forum', kwargs={
                'category': self.kwargs['category'],
                'forum': self.kwargs['forum']
            }
        )

    def get_context_data(self, **kwargs):
        context = super(EditTopicView, self).get_context_data(**kwargs)
        context['forum'] = self.kwargs['forum']
        context['category'] = self.kwargs['category']
        return context

    def get(self, request, category, forum, idtopic, *args, **kwargs):
        # Get topic
        topic = get_object_or_404(
            models.Topic, idtopic=idtopic, user_id=request.user.id
        )

        # Init fields form
        form = forms.FormEditTopic(instance=topic)

        data = {
            'form': form,
            'forum': forum,
            'topic': topic,
            'category': category,
        }

        return render(request, self.template_name, data)

    def post(self, request, category, forum, idtopic, *args, **kwargs):
        # Get topic
        topic = get_object_or_404(
            models.Topic, idtopic=idtopic, user_id=request.user.id
        )
        file_name = topic.attachment

        # Get form
        form = forms.FormEditTopic(request.POST, request.FILES, instance=topic)
        file_path = settings.MEDIA_ROOT

        if form.is_valid():

            obj = form.save(commit=False)

            title = conditional_escape(request.POST['title'])
            slug = defaultfilters.slugify(request.POST['title'])

            obj.title = title
            obj.slug = slug

            # If check field clear, remove file when update
            if 'attachment-clear' in request.POST:
                route_file = utils.get_route_file(file_path, file_name.name)

                try:
                    utils.remove_file(route_file)
                except Exception:
                    pass

            # If has attachment
            if 'attachment' in request.FILES:

                if not topic.id_attachment:
                    id_attachment = get_random_string(length=32)
                    obj.id_attachment = id_attachment

                file_name_post = request.FILES['attachment']
                obj.attachment = file_name_post

                # Route previous file
                route_file = utils.get_route_file(file_path, file_name.name)

                try:
                    # If a previous file exists it removed
                    utils.remove_file(route_file)
                except Exception:
                    pass

            # Update topic
            form.save()

            messages.success(
                request,
                _("The topic '%(topic)s' was successfully edited")
                % {'topic': obj.title}
            )
            return self.form_valid(form, **kwargs)
        else:
            messages.error(request, _("Invalid form"))
            return self.form_invalid(form, **kwargs)


class DeleteTopicView(mixins.UserTrollMixin, View):
    """
    This view will delete one topic
    """
    def delete(self, request, *args, **kwargs):

        # Dict delete
        qd = QueryDict(request.body)
        DELETE_DICT = {k: v[0] if len(v) == 1 else v for k, v in qd.lists()}

        # Get params
        forum = DELETE_DICT.get("forum")
        idtopic = DELETE_DICT.get("idtopic")

        # Previouly verify that exists the topic
        topic = get_object_or_404(
            models.Topic, idtopic=idtopic, user_id=request.user.id
        )

        # Get data topic
        iduser_topic = topic.user_id
        title_topic = topic.title

        # If my user so delete
        if request.user.id == iduser_topic:
            utils.remove_folder_attachment(idtopic)
            models.Topic.objects.filter(
                idtopic=idtopic, user_id=iduser_topic
            ).delete()
            messages.success(
                request, _("The topic '%(topic)s' was successfully deleted")
                % {'topic': title_topic}
            )
        else:
            raise HttpResponse(status=404)

        return HttpResponse(status=200)


class OpenCloseTopicView(mixins.UserTrollMixin, View):
    """
    This view close or re-open topic
    """
    def get(self, request, *args, **kwargs):
        return Http404()

    def post(self, request, *args, **kwargs):
        userid = int(request.POST.get("userid"))
        idtopic = int(request.POST.get("idtopic"))
        is_close = int(request.POST.get("is_close"))

        # Check if has params
        if idtopic and userid:
            # Only same user can close topic
            if request.user.id == userid:
                status = True if is_close == 1 else False
                # Close or re-open topic
                models.Topic.objects.filter(idtopic=idtopic).update(
                    is_close=status
                )

                return HttpResponse(status=200)
            else:
                return HttpResponse(status=404)
        else:
            return HttpResponse(status=404)


class LikeUnlikeTopicView(View):
    """
    This view like or unlike topic
    """
    def get(self, request, *args, **kwargs):
        return Http404()

    def post(self, request, *args, **kwargs):
        idtopic = int(request.POST.get("idtopic"))
        is_like = int(request.POST.get("is_like"))

        # Get topic
        topic = get_object_or_404(models.Topic, idtopic=idtopic)

        # If is like
        if is_like == 1:
            # Like topic
            models.Topic.objects.filter(idtopic=idtopic).update(
                like=F('like') + 1
            )
            # Add reference
            models.LikeTopic.objects.create(
                topic=topic, user=request.user
            )
        else:
            # Like topic
            models.Topic.objects.filter(idtopic=idtopic).update(
                like=F('like') - 1
            )
            # Delete reference
            models.LikeTopic.objects.filter(
                topic=topic, user=request.user
            ).delete()

        return HttpResponse(status=200)


class LikeUnlikeCommentView(View):
    """
    This view like or unlike comment
    """
    def get(self, request, *args, **kwargs):
        return Http404()

    def post(self, request, *args, **kwargs):
        idcomment = int(request.POST.get("idcomment"))
        is_like = int(request.POST.get("is_like"))

        # Get comment
        comment = get_object_or_404(models.Comment, idcomment=idcomment)

        # If is like
        if is_like == 1:
            # Like comment
            models.Comment.objects.filter(idcomment=idcomment).update(
                like=F('like') + 1
            )
            # Add reference
            models.LikeComment.objects.create(
                comment=comment, user=request.user
            )
        else:
            # Like comment
            models.Comment.objects.filter(idcomment=idcomment).update(
                like=F('like') - 1
            )
            # Delete reference
            models.LikeComment.objects.filter(
                comment=comment, user=request.user
            ).delete()

        return HttpResponse(status=200)


class NewCommentView(mixins.UserTrollMixin, View):
    """
    This view allowed add new comment to topic
    """
    def get(self, request, category, forum, slug, idtopic, *args, **kwargs):
        raise Http404()

    def post(self, request, category, forum, slug, idtopic, *args, **kwargs):
        # Form new comment
        form = forms.FormAddComment(request.POST)

        url = reverse_lazy('topic', kwargs={
            'category': category, 'forum': forum,
            'slug': slug, 'idtopic': str(idtopic)
        })

        if form.is_valid():
            obj = form.save(commit=False)

            # Save new comment
            now = timezone.now()
            User = get_user_model()
            user = User.objects.get(id=request.user.id)
            topic = get_object_or_404(models.Topic, idtopic=idtopic)
            obj.date = now
            obj.user = user
            obj.topic_id = topic.idtopic
            obj.save()

            # Update last activity TopicSearch
            models.Topic.objects.filter(
                idtopic=idtopic
            ).update(last_activity=now)

            # Data for notification real time
            idcomment = obj.idcomment
            comment = models.Comment.objects.get(idcomment=idcomment)
            username = request.user.username

            # Get photo profile
            photo = utils.get_photo_profile(request.user.id)

            # Send notifications comment
            params = utils.get_users_and_send_notification_comment(
                request, topic, comment
            )
            list_us = params['list_us']
            list_email = params['list_email']

            # Send email
            form.send_mail_comment(str(url), list_email)

            # Data necessary for realtime
            data = realtime.data_base_realtime(
                comment.topic, photo, forum, username
            )

            # Send new notification realtime
            realtime.new_notification(data, list_us)

            # Send new comment in realtime
            realtime.new_comment(data, comment.description)

            messages.success(request, _("Added new comment"))
            return HttpResponseRedirect(url)
        else:
            messages.error(request, _("Field required"))
            return HttpResponseRedirect(url)


class EditCommentView(mixins.UserTrollMixin, View):
    """
    This view allowed edit comment to topic
    """
    def get(self, request, category, forum, slug, idtopic, idcomment,
            *args, **kwargs):
        raise Http404()

    def post(self, request, category, forum, slug, idtopic, idcomment,
             *args, **kwargs):

        url = reverse_lazy('topic', kwargs={
            'category': category, 'forum': forum,
            'slug': slug, 'idtopic': str(idtopic)
        })

        # Valid if has description
        description = request.POST.get('update_description')
        if description:
            # Edit comment
            iduser = request.user.id
            models.Comment.objects.filter(
                idcomment=idcomment, user=iduser
            ).update(
                description=description
            )

            messages.success(request, _("Comment edited"))
            return HttpResponseRedirect(url)
        else:
            return HttpResponseRedirect(url)


class DeleteCommentView(mixins.UserTrollMixin, View):
    """
    This view allowed remove comment to topic
    """
    def get(self, request, category, forum, slug, idtopic, idcomment,
            *args, **kwargs):
        raise Http404()

    def post(self, request, category, forum, slug, idtopic, idcomment,
             *args, **kwargs):

        url = reverse_lazy('topic', kwargs={
            'category': category, 'forum': forum,
            'slug': slug, 'idtopic': str(idtopic)
        })

        # Delete comment and notification
        try:
            iduser = request.user.id
            models.Comment.objects.filter(
                idcomment=idcomment, user=iduser
            ).delete()

            messages.success(request, _("Comment deleted"))
            return HttpResponseRedirect(url)
        except Exception:
            return HttpResponseRedirect(url)


class AllNotification(View):
    """
    This view return all notification and paginate
    """
    def get(self, request, *args, **kwargs):
        template_name = "musette/all_notification_index.html"
        page_template = "musette/all_notification.html"

        iduser = request.user.id

        # Set all notification like view
        models.Notification.objects.filter(iduser=iduser).update(
            is_view=True
        )

        # Get all notification user
        notifications = utils.get_notifications(iduser)
        data = {
            'notifications': notifications,
        }

        if request.is_ajax():
            template_name = page_template
        return render(request, template_name, data)


def SetNotifications(request):
    """
    This view set all views notifications in true
    """
    iduser = request.user.id
    models.Notification.objects.filter(iduser=iduser).update(is_view=True)

    return HttpResponse("Ok")


class AddRegisterView(mixins.UserTrollMixin, View):
    """
    This view add register to forum
    """
    def get(self, request, category, forum, *args, **kwargs):
        raise Http404()

    def post(self, request, category, forum, *args, **kwargs):
        url = reverse_lazy(
            'forum', kwargs={'category': category, 'forum': forum}
        )

        # Get data
        forum = get_object_or_404(
            models.Forum, category__name=category, name=forum, hidden=False
        )
        idforum = forum.idforum
        iduser = request.user.id
        date = timezone.now()

        # Add new register
        register = models.Register(
            forum_id=idforum, user_id=iduser,
            date=date
        )
        register.save()
        messages.success(request, _("You have successfully registered"))
        return HttpResponseRedirect(url)


class UnregisterView(mixins.UserTrollMixin, View):
    """
    This view remove register to forum
    """
    def get(self, request, category, forum, *args, **kwargs):
        raise Http404()

    def post(self, request, category, forum, *args, **kwargs):
        url = reverse_lazy(
            'forum', kwargs={'category': category, 'forum': forum}
        )

        # Get data
        forum = get_object_or_404(
            models.Forum, category__name=category, name=forum, hidden=False
        )
        idforum = forum.idforum
        iduser = request.user.id

        # Remove register
        models.Register.objects.filter(
            forum_id=idforum, user_id=iduser,
        ).delete()

        messages.success(request, _("Registration was successfully canceled"))
        return HttpResponseRedirect(url)


class UsersForumView(View):
    """
    This view display users register in forum
    """
    def get(self, request, category, forum, *args, **kwargs):

        template_name = "musette/users_forum_index.html"
        page_template = "musette/users_forum.html"

        # Get register users
        forum = get_object_or_404(
            models.Forum, category__name=category, name=forum, hidden=False
        )
        moderators = forum.moderators.all()
        # Get registers, exclude moderators
        registers = forum.register_forums.filter(~Q(user__in=moderators))

        # Add moderator to users
        users = list(chain(registers, moderators))

        data = {
            'forum': forum,
            'users': users,
        }

        if request.is_ajax():
            template_name = page_template
        return render(request, template_name, data)

    def post(self, request, forum, *args, **kwargs):
        raise Http404()


class TopicSearch(View):
    """
    This view django, display results of search of topics
    """
    def get(self, request, category, forum, *args, **kwargs):
        template_name = "musette/topic_search_index.html"
        page_template = "musette/topic_search.html"

        # Get param to search
        search = request.GET.get('q')

        # Get id forum
        forum = get_object_or_404(
            models.Forum, category__name=category, name=forum
        )
        idforum = forum.idforum

        # Search topics
        topics = models.Topic.objects.filter(
            forum_id=idforum, title__icontains=search
        )

        data = {
            'topics': topics,
            'forum': forum,
        }

        if request.is_ajax():
            template_name = page_template
        return render(request, template_name, data)


class ProfileView(View):
    """
    This view django, display results of the profile
    """
    def get(self, request, username, *args, **kwargs):
        template_name = "musette/profile.html"

        # Get user param
        User = get_user_model()
        user = get_object_or_404(User, username=username)
        iduser = user.id

        # Get model extend Profile
        ModelProfile = utils.get_main_model_profile()

        # Get name app of the extend model Profile
        app = utils.get_app_model(ModelProfile)

        # Check if the model profile is extended
        count_fields_model = utils.get_count_fields_model(ModelProfile)
        count_fields_abstract = utils.get_count_fields_model(
            models.AbstractProfile
        )
        if count_fields_model > count_fields_abstract:
            model_profile_is_extend = True
        else:
            model_profile_is_extend = False
        profile = get_object_or_404(ModelProfile, iduser=iduser)

        photo = utils.get_photo_profile(iduser)

        # Get last topic of the profile
        topics = models.Topic.objects.filter(user=user)[:5]

        data = {
            'profile': profile,
            'photo': photo,
            'user': request.user,
            'topics': topics,
            'app': app,
            'model_profile_is_extend': model_profile_is_extend
        }

        return render(request, template_name, data)


class EditProfileView(mixins.UserTrollMixin, FormView):
    """
    This view allowed edit profile
    """
    template_name = "musette/edit_profile.html"
    form_class = forms.FormEditProfile

    def get_success_url(self):
        return reverse_lazy('profile', kwargs={
            'username': self.kwargs['username']
        })

    def get(self, request, username, *args, **kwargs):

        ModelProfile = utils.get_main_model_profile()
        profile = get_object_or_404(
            ModelProfile, iduser=request.user.id
        )

        # Init fields form
        form = forms.FormEditProfile(instance=profile)

        data = {
            'form': form
        }

        return render(request, self.template_name, data)

    def post(self, request, username, *args, **kwargs):

        ModelProfile = utils.get_main_model_profile()
        profile = get_object_or_404(
            ModelProfile, iduser=request.user.id
        )

        file_name = profile.photo

        form = forms.FormEditProfile(
            request.POST, request.FILES, instance=profile
        )
        file_path = settings.MEDIA_ROOT

        if form.is_valid():

            obj = form.save(commit=False)
            about = request.POST['about']
            obj.about = about

            # If check field clear, remove file when update
            if 'attachment-clear' in request.POST:
                route_file = utils.get_route_file(file_path, file_name.name)

                try:
                    utils.remove_file(route_file)
                except Exception:
                    pass

            if 'attachment' in request.FILES:

                if not obj.id_attachment:
                    id_attachment = get_random_string(length=32)
                    obj.id_attachment = id_attachment

                file_name_post = request.FILES['photo']
                obj.photo = file_name_post

                # Route previous file
                route_file = utils.get_route_file(file_path, file_name.name)

                try:
                    # If a previous file exists it removed
                    utils.remove_file(route_file)
                except Exception:
                    pass

            # Update profile
            form.save()

            messages.success(
                request,
                _("Your profile was successfully edited")
            )
            return self.form_valid(form, **kwargs)
        else:
            messages.error(request, _("Invalid form"))
            return self.form_invalid(form, **kwargs)


class IsTrollView(mixins.UserTrollMixin, View):
    """
    Set if a user is troll
    """
    def get(self, request, *args, **kwargs):
        raise Http404()

    def post(self, request, *args, **kwargs):
        user_post = request.POST.get('username')
        check = True if int(request.POST.get('check')) == 1 else False

        # Get troll
        User = get_user_model()
        username = get_object_or_404(User, username=user_post)

        # Check if is user correct
        total = utils.get_total_forum_moderate_user(username)
        if not username.is_superuser and total == 0:
            # Is a troll
            ModelProfile = utils.get_main_model_profile()
            ModelProfile.objects.filter(iduser=username).update(is_troll=check)

        return redirect(
            "profile", username=user_post
        )



from __future__ import print_function

import json
import threading
from functools import partial

import redis
import tornado.httpserver
import tornado.ioloop
import tornado.web
import tornado.websocket

# Client users listener list
clients_notifications = []
clients_comments = []


def redis_listener_notification():
    """
    This method subscribe to redis and send message
    to handler websocket notifications
    """
    r = redis.Redis()
    ps = r.pubsub()
    ps.subscribe('notifications')
    io_loop = tornado.ioloop.IOLoop.instance()

    # Listen notifications
    for message in ps.listen():
        for element in clients_notifications:
            data = json.loads(message['data'].decode("utf-8"))
            if 'list_us' in data:
                users = data['list_us']
                user = int(element.user)
                if user in users:
                    io_loop.add_callback(partial(element.on_message, message))
                else:
                    continue


def redis_listener_comment():
    """
    This method subscribe to redis and send message
    to handler websocket comments
    """
    r = redis.Redis()
    ps = r.pubsub()
    ps.subscribe('comments')
    io_loop = tornado.ioloop.IOLoop.instance()

    # Listen comments
    for message in ps.listen():
        for element in clients_comments:
            data = json.loads(message['data'].decode("utf-8"))
            if 'idtopic' in data:
                idtopic = data['idtopic']
                topic = int(element.topic)
                if topic == idtopic:
                    io_loop.add_callback(partial(element.on_message, message))
                else:
                    continue


class RealtimeHandler(tornado.websocket.WebSocketHandler):
    """
    Handler websocket
    """
    def check_origin(self, origin):
        return True

    def open(self):
        self.user = self.get_argument('user', None)
        self.topic = self.get_argument('topic', None)

        if self.user:
            print('New connection was opened. User: ' + self.user)
            clients_notifications.append(self)
        elif self.topic:
            print('New connection was opened. Topic: ' + self.topic)
            clients_comments.append(self)

    def on_message(self, message):
        self.write_message(message['data'])

    def on_close(self):
        print('Conn closed...')
        if self.user:
            clients_notifications.remove(self)
        elif self.topic:
            clients_comments.remove(self)


# Settings for server tornado
settings = {
    'auto_reload': True,
}

# Routes application tornado
application = tornado.web.Application([
    (r'/ws/', RealtimeHandler),
], **settings)


if __name__ == "__main__":
    # Thread for function redis_listener_notification
    threading.Thread(target=redis_listener_notification).start()
    # Thread for function redis_listener_comment
    threading.Thread(target=redis_listener_comment).start()
    # Run server tornado
    http_server = tornado.httpserver.HTTPServer(application)
    http_server.listen(8888)
    tornado.ioloop.IOLoop.instance().start()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-02-06 03:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('musette', '0005_likecomment_liketopic'),
    ]

    operations = [
        migrations.AlterField(
            model_name='topic',
            name='is_close',
            field=models.BooleanField(default=False, help_text='If the topic is closed', verbose_name='Closed topic'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-01-27 00:55
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('musette', '0004_auto_20170126_2118'),
    ]

    operations = [
        migrations.CreateModel(
            name='LikeComment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likes_comment', to='musette.Comment')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likes_comment_users', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='LikeTopic',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('topic', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likes_topic', to='musette.Topic')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likes_topic_users', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.utils.decorators import method_decorator
from musette.decorators import user_is_troll


class UserTrollMixin(object):
    """
    Mixin for check if the user is a troll
    """
    @method_decorator(user_is_troll)
    def dispatch(self, *args, **kwargs):
        return super(UserTrollMixin, self).dispatch(*args, **kwargs)



from rest_framework import routers
from musette.api import views

# Routers provide a way of automatically determining the URL conf.
router = routers.DefaultRouter()
router.register(r'api/users', views.UserViewSet)
router.register(r'api/categories', views.CategoryViewSet)
router.register(r'api/forums', views.ForumViewSet)
router.register(r'api/topics', views.TopicViewSet)
router.register(r'api/registers', views.RegisterViewSet)
router.register(r'api/comments', views.CommentViewSet)
router.register(r'api/profiles', views.ProfileViewSet)



from django import forms


class TextareaWidget(forms.Textarea):
    """
    Widget rich textarea
    """
    class Media:
        # tiny_mce
        js = ('//cdn.tinymce.com/4/tinymce.min.js',
              '/static/musette/js/textareas.js')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-03-04 22:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('musette', '0006_auto_20170206_0017'),
    ]

    operations = [
        migrations.AddField(
            model_name='configuration',
            name='description',
            field=models.TextField(blank=True, verbose_name='Description'),
        ),
    ]



from django.conf import settings

SITE_NAME = getattr(settings, "SITE_NAME", "Musette")

# Number of seconds of inactivity before a user is marked offline
USER_ONLINE_TIMEOUT = 300

# Number of seconds that we will keep track of inactive users for before
# their last seen is removed from the cache
USER_LASTSEEN_TIMEOUT = 60 * 60 * 24 * 7



from django.db.models.signals import m2m_changed, pre_delete, post_save
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.dispatch import receiver

from musette import models, utils


@receiver(post_save, sender=settings.AUTH_USER_MODEL)
def post_save_user(sender, instance, **kwargs):
    """
    This signal is event of model user for create new profile
    """
    if kwargs['created']:
        subclasses = models.AbstractProfile.__subclasses__()
        if len(subclasses) > 0:
            User = get_user_model()
            user = User.objects.get(id=instance.id)
            Profile = subclasses[0]

            # For confirm email
            data = utils.get_data_confirm_email(user.email)

            # Insert profile
            profile = Profile(
                iduser=user, photo="", about="",
                activation_key=data['activation_key'],
                key_expires=data['key_expires']
            )
            profile.save()

            # Send email for confirm user
            utils.send_welcome_email(
                user.email, user.username, data['activation_key']
            )


@receiver(m2m_changed, sender=models.Forum.moderators.through)
def post_save_forum(sender, instance, **kwargs):
    """
    This signal is event of model forum for add permissions
    """
    # If add moderator
    if kwargs['action'] == 'post_add':
        for moderator in instance.moderators.all():
            # Superuser not is necessary
            if not moderator.is_superuser:
                instance.add_permissions_topic_moderator(moderator)

    # If remove moderator
    elif kwargs['action'] == 'post_remove':
        ids_removed = kwargs['pk_set']
        for id in ids_removed:
            User = get_user_model()
            old_moderator = User.objects.get(id=id)
            # Superuser not is necessary
            if not old_moderator.is_superuser:
                # Only remove permissions if moderator has one forum
                if instance.tot_forums_moderators(old_moderator) < 1:
                    instance.clear_permissions_moderator(old_moderator)


@receiver(pre_delete, sender=models.Topic)
@receiver(pre_delete, sender=models.Comment)
def pre_delete_receiver_notification(sender, instance, **kwargs):
    """
    Delete notification comment or topic
    """
    ctype = ContentType.objects.get_for_model(instance)
    models.Notification.objects.filter(
        content_type=ctype, idobject=instance.pk
    ).delete()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-01-25 00:56
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('musette', '0002_auto_20170124_1509'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='forum',
            unique_together=set([('category', 'name')]),
        ),
    ]



from django.conf import settings

from .models import Configuration
from .utils import get_notifications


def data_templates(request):
    """
    context_processors for get in all templates
    """
    # Get notifications data
    notifications = get_notifications(request.user.id)

    # Get configurations
    try:
        configurations = Configuration.objects.all()[:1].get()
    except Configuration.DoesNotExist:
        configurations = None

    return {
        'SETTINGS': settings,
        'notifications': notifications,
        'configurations': configurations
    }



from django.db.models import Q
from django.contrib.auth import get_user_model

from rest_framework import serializers
from musette import models, utils


# Serializers Users
class UserSerializer(serializers.ModelSerializer):

    class Meta:
        model = get_user_model()
        fields = (
            'username', 'is_superuser', 'first_name', 'last_name',
            'email', 'is_staff', 'is_active', 'date_joined'
        )


# Serializers Categories
class CategorySerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Category
        fields = '__all__'


# Serializers Forum
class ForumSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Forum
        fields = '__all__'


# Serializers Topic
class TopicSerializer(serializers.ModelSerializer):

    def __init__(self, *args, **kwargs):
        super(TopicSerializer, self).__init__(*args, **kwargs)
        user = self.context['request'].user

        # If no is superuser, only forum register or is moderator
        if not user.is_superuser and user.is_authenticated():
            registers = models.Register.objects.filter(user=user)
            self.fields['forum'].queryset = None
            self.fields['forum'].queryset = models.Forum.objects.filter(
                Q(moderators__in=[user.id]) | Q(register_forums__in=registers)
            ).distinct()

            # Only my user
            User = get_user_model()
            self.fields['user'].queryset = User.objects.filter(id=user.id)

    class Meta:
        model = models.Topic
        exclude = (
            'slug', 'date', 'moderate', 'id_attachment', 'is_top',
        )


# Serializers register
class RegisterSerializer(serializers.ModelSerializer):

    def __init__(self, *args, **kwargs):
        super(RegisterSerializer, self).__init__(*args, **kwargs)
        user = self.context['request'].user
        # If no is superuser, get forum that
        # not is register or not is moderator
        if not user.is_superuser and user.is_authenticated():
            registers = models.Register.objects.filter(user=user)
            self.fields['forum'].queryset = models.Forum.objects.filter(
                ~Q(moderators__in=[user.id]), ~Q(
                    register_forums__in=registers
                )
            )

            # Only my user
            User = get_user_model()
            self.fields['user'].queryset = User.objects.filter(id=user.id)

    class Meta:
        model = models.Register
        exclude = ('date',)


# Serializers comment
class CommentSerializer(serializers.ModelSerializer):

    def __init__(self, *args, **kwargs):
        super(CommentSerializer, self).__init__(*args, **kwargs)
        user = self.context['request'].user
        if not user.is_superuser and user.is_authenticated():
            # Only my user
            User = get_user_model()
            self.fields['user'].queryset = User.objects.filter(id=user.id)

            # Topic not is close and is moderate not show
            self.fields['topic'].queryset = models.Topic.objects.filter(
                is_close=False, moderate=True
            )

    class Meta:
        model = models.Comment
        exclude = ('date',)


# Serializers profile
class ProfileSerializer(serializers.ModelSerializer):

    class Meta:
        model = utils.get_main_model_profile()
        fields = '__all__'



from rest_framework import permissions
from musette import utils


class IsReadOnly(permissions.BasePermission):
    """
    Readonly permissions
    """
    def has_permission(self, request, view):
        if request.method in permissions.SAFE_METHODS:
            return True


class ForumPermissions(permissions.BasePermission):
    """
    Check if is superuser for can crete, remove, etc.
    """
    def has_object_permission(self, request, view, obj):
        # Allow get requests for all
        if request.method == 'GET':
            return True
        else:
            # Get is moderator forum
            if hasattr(obj, 'forum'):
                forum = obj.forum
                category = obj.forum.category.name
            elif hasattr(obj, 'topic'):
                forum = obj.topic.forum
                category = obj.topic.forum.category.name
            else:
                forum = None
                category = None

            # Get if is moderator
            is_moderator = utils.is_user_moderator_forum(
                category, forum, request.user
            )

            # Only allow if is superuser or moderator or creted topic
            return (
                request.user.is_superuser or
                is_moderator or
                obj.user == request.user
            )



#!/usr/bin/env python
import os
import sys
from setuptools import setup

sys.path.insert(0, 'musette')
from version import get_version
sys.path.remove('musette')

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name='django-musette',
    version=get_version(),
    packages=[
        'musette', 'musette.templatetags', 'musette.migrations',
        'musette.websockets', 'musette.management',
        'musette.management.commands', 'musette.api',
    ],
    include_package_data=True,
    license='BSD License',
    zip_safe=False,
    description='Forum for Django framework.',
    long_description=README,
    url='https://github.com/mapeveri/django-musette',
    author='Peveri Martin',
    author_email='martinpeveri@gmail.com',
    install_requires=[
        'Django>=1.8',
        'tornado==4.2',
        'django-redis-cache==1.6.5',
        'djangorestframework==3.5.4',
        'django-endless-pagination-vue==1.3',
        'django-hitcount==1.2.2'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.http import Http404
from django.utils import timezone

from . import settings


class ActiveUserMiddleware(object):
    """
    Set user authenticate
    """
    def process_request(self, request):
        current_user = request.user
        if request.user.is_authenticated():
            now = timezone.now()
            cache.set(
                'seen_%s' % (current_user.username), now,
                settings.USER_LASTSEEN_TIMEOUT
            )


class RestrictStaffToAdminMiddleware(object):
    """
    A middleware that restricts staff members
    access to administration panels
    """
    def process_request(self, request):
        if request.path.startswith(reverse('admin:index')):
            if request.user.is_authenticated():
                if not request.user.is_staff:
                    raise Http404
            else:
                raise Http404



from django.apps import AppConfig


class MusetteConfig(AppConfig):
    """
    App configuration
    """
    name = 'musette'

    def ready(self):
        # Import signals
        import musette.signals



import json
import redis

from django.conf import settings


def data_base_realtime(obj, photo, forum, username):
    """
    Get data comment for new topic or new comment
    """
    # Data necessary for realtime
    data = {
        "topic": obj.title,
        "idtopic": obj.idtopic,
        "slug": obj.slug,
        "settings_static": settings.STATIC_URL,
        "username": username,
        "forum": forum,
        "category": obj.forum.category.name,
        "photo": photo
    }

    return data


def new_notification(data_notification, list_us):
    """
    Send new notification topic or comment to redis
    """
    # Add item for notification
    data_notification['list_us'] = list_us

    # Add to real time new notification
    json_data_notification = json.dumps(data_notification)
    # Redis instance
    r = redis.StrictRedis()
    # Publish
    r.publish('notifications', json_data_notification)


def new_comment(data_comment, comment_description):
    """
    Send new comment
    """
    # Publish new comment in topic
    data_comment['description'] = comment_description
    json_data_comment = json.dumps(data_comment)
    # Redis instance
    r = redis.StrictRedis()
    # Publish
    r.publish('comments', json_data_comment)



import inspect
import os

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Run server tornado web sockets."

    def handle(self, *args, **options):
        self.stdout.write('Running server...')
        path = os.path.dirname(
            os.path.abspath(inspect.getfile(inspect.currentframe())))
        path = os.path.dirname(os.path.dirname(path))
        self.stdout.write('Tornado server initialized')
        os.system("python " + path + "/websockets/server.py")



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-01-27 00:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('musette', '0003_auto_20170124_2156'),
    ]

    operations = [
        migrations.AddField(
            model_name='comment',
            name='like',
            field=models.PositiveIntegerField(default=0, editable=False),
        ),
        migrations.AddField(
            model_name='topic',
            name='like',
            field=models.PositiveIntegerField(default=0, editable=False),
        ),
    ]



from django import template
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse_lazy
from django.utils import formats, timezone

from hitcount.models import HitCount

from ..models import Comment, Forum, Topic, Notification
from .photo import get_photo
from ..utils import (
    get_photo_profile, get_datetime_topic,
    get_total_forum_moderate_user
)

register = template.Library()


@register.filter
def in_category(category):
    """
    This tag filter the forum for category
    """
    return Forum.objects.filter(
        category_id=category,
        hidden=False
    )


@register.filter
def get_tot_comments(idtopic):
    """
    This tag filter return the total
    comments of one topic
    """
    return Comment.objects.filter(
        topic_id=idtopic,
    ).count()


@register.simple_tag
def get_tot_views(idtopic):
    """
    This tag filter return the total
    views for topic or forum
    """
    try:
        content = Topic.objects.get(idtopic=idtopic)
        idobj = ContentType.objects.get_for_model(content)

        hit = HitCount.objects.get(
            object_pk=idtopic,
            content_type_id=idobj
        )
        total = hit.hits
    except Exception:
        total = 0

    return total


@register.filter
def get_path_profile(user):
    """
    Return tag a with profile
    """
    username = getattr(user, "username")
    url = str(reverse_lazy("profile", kwargs={'username': username}))
    tag = "<a href='" + url + "'>" + username + " </a>"

    return tag


@register.filter
def get_tot_users_comments(topic):
    """
    This tag filter return the total
    users of one topic
    """
    idtopic = topic.idtopic
    users = Comment.objects.filter(topic_id=idtopic)

    data = ""
    lista = []
    for user in users:
        username = user.user.username
        url = str(reverse_lazy("profile", kwargs={'username': username}))

        if not (username in lista):
            lista.append(username)

            photo = get_photo(user.user.id)

            tooltip = "data-toggle='tooltip' data-placement='bottom' "
            tooltip += "title='" + username + "'"
            data += "<a href='" + url + "' " + tooltip + ">"
            data += "<img class='img-circle' src='" + str(photo) + "' "
            data += "width=30, height=30></a>"

    if len(users) == 0:
        username = topic.user.username
        url = str(reverse_lazy("profile", kwargs={'username': username}))
        iduser = topic.user.id

        photo = get_photo(iduser)
        data += "<a href='" + url + "'>"
        data += "<img class='img-circle' src='" + str(photo) + "' "
        data += "width=30, height=30></a>"

    return data


@register.filter
def get_tot_topics_moderate(forum):
    """
    This filter return info about
    Few topics missing for moderate
    """
    topics_count = forum.topics_count
    idforum = forum.idforum

    moderates = Topic.objects.filter(
        forum_id=idforum,
        moderate=True).count()
    return topics_count - moderates


@register.filter
def get_item_notification(notification):
    """
    This filter return info about
    one notification of one user
    """
    idobject = notification.idobject
    is_comment = notification.is_comment

    html = ""
    try:
        # If is comment notification
        if is_comment:
            comment = Comment.objects.get(idcomment=idobject)
            forum = comment.topic.forum.name
            category = comment.topic.forum.category.name
            slug = comment.topic.slug
            idtopic = comment.topic.idtopic
            username = comment.user.username
            title = comment.topic.title
            userid = comment.user.id
        else:
            # Is topic notification
            topic = Topic.objects.get(idtopic=idobject)
            forum = topic.forum.name
            category = topic.forum.category.name
            slug = topic.slug
            idtopic = topic.idtopic
            username = topic.user.username
            title = topic.title
            userid = topic.user.id

        url_topic = str(reverse_lazy("topic", kwargs={
            'category': category, 'forum': forum,
            'slug': slug, 'idtopic': idtopic
        }))
        title = "<h5><a href='" + url_topic + "'><u>" + title + "</u></a></h5>"

        # Data profile
        photo = get_photo_profile(userid)
        date = get_datetime_topic(notification.date)
        url_profile = str(reverse_lazy("profile", kwargs={'username': username}))

        # Notificacion
        html += '<a class="content" href="' + url_topic + '">'
        html += ' <h4 class="item-title">'
        html += ' <img class="img-circle pull-left" src="' + photo + '"'
        html += ' width=45 height=45 />'
        html += title + '</h4>'
        html += ' <p class="item-info"><a href="' + url_profile + '">'
        html += username + "</a> - " + str(date)
        html += '</p></a>'
    except Comment.DoesNotExist:
        html = ""

    return html


@register.filter
def get_pending_notifications(user):
    """
    This method return total pending notifications
    """
    return Notification.objects.filter(
        is_view=False, iduser=user).count()


@register.filter
def get_last_activity(topic):
    """
    This method return last activity of topic
    """
    # Get timezone for datetime
    d_timezone = timezone.localtime(topic.last_activity)
    # Format data
    date = formats.date_format(d_timezone, "SHORT_DATETIME_FORMAT")

    # Return format data more user with tag <a>
    html = ""
    # html += get_path_profile(topic.user)
    html += " <p>" + str(date) + "</p>"
    return html


@register.filter
def get_object_user(obj, user):
    """
    Get object user
    """
    if obj:
        return user.user
    else:
        return user


@register.filter
def get_tot_users_forum(forum):
    """
    Get total users register and moderators
    """
    users_registers = forum.register_forums.all().count()
    moderators = forum.moderators.all().count()
    return users_registers + moderators


@register.filter
def check_like_comment(comment, user):
    """
    Check if like comment is checked
    """
    return user.likes_comment_users.filter(
        user=user, comment=comment
    ).count() == 0


@register.filter
def check_like_topic(topic, user):
    """
    Check if like topic is checked
    """
    return user.likes_topic_users.filter(
        user=user, topic=topic
    ).count() == 0


@register.filter
def get_total_forum_moderator(user):
    """
    Get total of forums that moderate one user
    """
    return get_total_forum_moderate_user(user)
